unit ACOVARS; {Unit 8 of OVERSAMPLE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, SCREEN, POST, Graph, Dos, Crt;

  procedure ARMACovariances (var alpha, mu: vector;
                  var gamma: vector;
                  var varEpsilon: real;
                  var lags, p, q: integer);

  procedure ARMACorrelations (var lags: integer;
                  var rho: vector;
                  var gamma: vector;
                  var autoMin: real);

  procedure CorrelPlot (plotOneOrTwo: integer;
                  var ARMASpec: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var CorrelBOX: rBOX;
                  var CorrelVec, secondVec: longVector;
                  var lags: integer);

implementation

{_________________________________________________}

  procedure ARMACovariances (var alpha, mu: vector;
                  var gamma: vector;
                  var varEpsilon: real;
                  var lags, p, q: integer);

    var
      i, j, t, tau, r: integer;
      delta, f: vector;
      a: matrix;

  begin {Find the delta values}

    r := Max(p, q);
    for t := 0 to q do
      begin
        delta[t] := mu[t] * varEpsilon;
        tau := Min(t, p);
        for i := 1 to tau do
          delta[t] := delta[t] - delta[t - i] * alpha[i];
        delta[t] := delta[t] / alpha[0]
      end;

    for i := 0 to r do
      for j := 0 to r do
        begin {i,j : form the matrix of alpha values}
          a[i, j] := 0.0;
          if ((i - j) >= 0) and ((i - j) <= p) then
            a[i, j] := alpha[i - j];
          if ((i + j) <= p) and (j > 0) then
            a[i, j] := a[i, j] + alpha[i + j];
        end; {i, j}

    for i := 0 to r do
      begin{i : form the RHS vector}
        f[i] := 0.0;
        for j := i to q do
          f[i] := f[i] + mu[j] * delta[j - i]
      end; {i}

{Solve for the initial autocovariances}
    LUsolve(0, r + 1, a, gamma, f);

{Find the succeeding autocovariances}
    for i := r + 1 to lags do
      begin {i}
        gamma[i] := 0.0;
        for j := 1 to p do
          gamma[i] := gamma[i] - alpha[j] * gamma[i - j];
        gamma[i] := gamma[i] / alpha[0];
      end; {i}

  end; {ARMACovariances}

{_________________________________________________}

  procedure ARMACorrelations (var lags: integer;
                  var rho: vector;
                  var gamma: vector;
                  var autoMin: real);

    var
      i: integer;
  begin
    autoMin := 0.0;
    for i := 1 to lags do
      begin
        rho[i] := gamma[i] / gamma[0];
        if rho[i] < autoMin then
          autoMin := rho[i];
      end;
{autoMin := Abs(autoMin);}
    rho[0] := 1;

  end; {ARMACorrelations}

{_________________________________________________}

  procedure CorrelPlot (plotOneOrTwo: integer;
                  var ARMASpec: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var correlBOX: rBOX;
                  var correlVec, secondVec: longVector;
                  var lags: integer);
    var
      F: text;
      postFileName, legend: string;

  begin
    with ARMASpec do
      begin {ARMASpec}

        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        ScreenFrame(CorrelBOX, SCREENBOX);
        {SetLineStyle(solidLn, 0, thickWidth);}
        ScreenPlotMod(CorrelBOX, SCREENBOX, CorrelVec);
        {SetLineStyle(solidLn, 0, normWidth);}
        if (plotOneOrTwo = plotTwo) then
          begin
           SetColor(red);
           ScreenPlotMod(CorrelBOX, SCREENBOX, secondVec);
           SetColor(yellow);
           legend :='The continuous autocorrelation function of the specified process';
           legend := Concat(legend, ' and that of the estimated model (red line)');
           ScreenCaption(legend);
          end
        else
          SpikePlot(lags, CorrelBOX, SCREENBOX, rho);

        CallForReturn;
        Readln;
        CloseGraph;
        RestoreCrtMode;
        ClrScr;

        if PSPict then
          begin
            ClrScr;
            legend := 'A continuous autocovariance function and its sampled ordinates';

            Write('    Name the PostScript File = ');
            Readln(postFileName);
            if postFileName <> '' then
              begin {Postfile}
                Assign(F, postFileName);
                Rewrite(F);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrame(F, CorrelBOX, POSTBOX);
                PostPlotMod(F, CorrelBOX, POSTBOX, CorrelVec);

                if (plotOneOrTwo = plotTwo) then
                  PostPlotMod(F, CorrelBOX, POSTBOX, secondVec)
                else
                  PostSpikePlot(F, lags, CorrelBOX, POSTBOX, rho);

                PostFooter(F, POSTBOX, legend, TeXFormat);
                Close(F);
              end; {postFileName}
          end; {PSPict}

      end; {ARMASpec}

  end; {CorrelPlot }
{_________________________________________________}

end.  {ACOVARS: Unit 8 of OVERSAMPLE.PAS}

