unit GLOBALS; {Unit 1 of OVERSAMPLE.PAS}

interface

  const
    maxOrder = 12;
    maxArray = 300;
    halfmaxArray = 300;

  const {ASCII Codes}
    TAB = 9;
    CR = 13;
    SPACE = 32;

  const {Filter Types}
    Annual = 1;
    Quarterly = 4;
    Monthly = 12;
    Other = 9;

  const
    plotOne = 1;
    plotTwo = 2;

  type
    complex = record
        re, im: real;
      end;

  type
    polar = record
        argu, modu: real;
      end;

  type
    points = record
        x, y: real
      end;

  type
    vector = array[-2..200] of real;

  type
    precisionVector = array[-2..10] of double;

  type
    ivector = array[-10..50] of integer;

  type
    jVector = array[-20..20] of real;

  type
    longVector = array[-2..maxArray] of real;

  type
    pointsVector = array[0..halfmaxArray] of points;

  type
    complexVector = array[-20..20] of complex;

  type
    polarVector = array[-20..20] of polar;

  type
    fourierVec = array[0..halfmaxArray] of real;

  type
    matrix = array[0..25] of array[0..25] of real;

  type
    IBOX = record
        X0: integer;  {the horizontal value of the graphics origin}
        Y0: integer;  {the vertical value of the graphics origin}
        X1: integer; {left of the graphics bounding box}
        Y1: integer; {top or bottom of the graphics bounding box}
        X2: integer; {right of the graphics bounding box}
        Y2: integer; {bottom or top of graphics the bounding box}
        XC: integer; {horizontal value of the centre of window}
        YC: integer; {vertical value of the centre of window}
        xAxis: boolean;
        yAxis: boolean;
        doFrame: boolean;
        doMarks: boolean;
        lollipops: boolean;
      end;

  type
    rBOX = record
        xMax: real;  {the maximum value of x in the dataVector}
        xMin: real;  {the minimum value of x in the dataVector}
        yMax: real;  {the maximum value of y in the dataVector}
        yMin: real;  {the minimum value of y in the dataVector}
        xMean: real; {the mean value of x in the dataVector}
        yMean: real; {the mean value of y in the dataVector}
        xVar: real; {the variance of x in the dataVector}
        yVar: real; {the variance of y in the dataVector}
        yUB: real;   {yUB >= yMax: an upper bound for y}
        yLB: real;   {yLB <= yMin: a lower bound for y}
        xLB: real;   {xLB <= xMin: a left (lower) bound for x}
        xRB: real;    {xRB >= xMax: a right (upper) bound for x}
        NOPUNCT: integer; {number of data points}
        dataFrequency: integer; {Annual, Quarterly, Monthly, Other}
      end;

  type
    ARMARecord = record
        bandLimits: boolean; {have the limits been declared?}
        contaminated: boolean;
        ordersSpecified: boolean;
        parametersFound: boolean;
        autocovariancesFound: boolean;
        rootsFound: boolean;
        spectrumFound: boolean;
        sampleRate: integer; {the rate of oversampling}
        omega2Deg: real; {upper bound of spectral band}
        varEpsilon: real; {variance of white-noise forcing function}
        noiseVar: real; {variance of the white-noise contamination}
        p: integer; {degree of the denominator}
        pr: integer; {no of complex denominator roots}
        q: integer; {degree of the numerator}
        qr: integer; {no of complex numerator roots}
        alpha: vector; {coefficents of the denominator}
        mu: vector; {coefficents of the numerator}
        gamma: vector; {parametric autocovarainces}
        rho: vector; {autocorrelations}
        poles: complexVector; {Cartesian denominator roots}
        zeros: complexVector; {Cartesian numerator roots}
        polarPoles: polarVector; {polar denominator roots}
        polarZeros: polarVector; {polar numerator roots}
      end;

  {Graphics Prameters}
  var
    SCREENBOX, POSTBOX: IBOX;
    graphicsOn, noSCREENBOX: boolean;
    currentMode: integer;
    figNumber: integer;

   {Global Flags}
  var
    PSPict, TeXFormat: boolean;
    allsDone, noPageParameters: boolean;
    stability: boolean;

  var
    ARMASpec, ARMAEst: ARMARecord;
    spectrumVec: longVector;
    spectrumBOX: rBOX;

  procedure InitialiseARMARecord (var theRecord: ARMARecord);

implementation

{________________________________________________}

  procedure InitialiseARMARecord (var theRecord: ARMARecord);

    var
      j: integer;

  begin
    with theRecord do
      begin
        bandLimits := false; {have the limits been declared?}
        contaminated := false;
        ordersSpecified := false;
        parametersFound := false;
        autocovariancesFound := false;
        rootsFound := false;
        spectrumFound := false;
        sampleRate := 1; {the rate of oversampling}
        omega2Deg := 180; {upper bound of spectral band}
        varEpsilon := 1.0; {variance of white-noise forcing function}
        noiseVar := 0.0; {variance of the white-noise contamination}
        p := 0; {degree of the denominator}
        pr := 0; {no of complex denominator roots}
        q := 0; {degree of the numerator}
        qr := 0; {no of complex numerator roots}

{alpha: vector;}
 {coefficents of the denominator}
{mu: vector; }
{coefficents of the numerator}
{gamma: vector;}
 {parametric autocovarainces}
{rho: vector;}
 {autocorrelations}
{poles: complexVector;}
 {Cartesian denominator roots}
{zeros: complexVector;}
 {Cartesian numerator roots}
{polarPoles: polarVector;}
{polar denominator roots}
{polarZeros: polarVector;}
 {polar numerator roots}
      end; {with theRecord}

  end; {InitialiseARMARecord}

{_________________________________________________}

end. {GLOBALS: Unit 1 of OVERSAMPLE.PAS}

