program OVERSAMPLE;

  uses
    GLOBALS, MATHS, UTILS, ACQUIRE, ORGANISE, ACOVARS, Graph, Dos, Crt;

  var
    theMenuItem: integer;
 
begin
  SetTheText;
  Writeln;
  Writeln;
  Writeln('  OVERSAMPLE.PAS');
  Writeln;
  Writeln('  This program is for investigating the consequences of an over-rapid sampling');
  Writeln('  of a continuous-time process  generated by an ARMA model. It is assumed that');
  Writeln('  the forcing function of the model is a band-limited white-noise process with');
  Writeln('  a maximum frequency of pi radians per sample interval.');
  Writeln;
  Writeln('  The program generates the continuous-time autocovariance function from which');
  Writeln('  the discrete-time autocovariances corresponding to a specified sampling rate');
  Writeln('  can  be obtained. The autocovariances can be used to obtain the estimates of');
  Writeln('  the parameters of a misspecified ARMA model.');
  Writeln;
  Writeln('  It may be observed that data that are sampled too rapidly  will be supported');
  Writeln('  on a subset of the Nyquist frequency interval and that the resulting process');
  Writeln('  can be represented exactly only via an infinite-order ARMA model.');
  Writeln;
  Writeln(' ' : 70, '<RETURN>');
  Readln;

  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;

  allsDone := false;
  noPageParameters := true;
  stability := true; {we wish to impose MA roots of zero}

  stability := false; {This value is imposed with RationalAcquistion}
  figNumber := 1;

  InitialiseARMARecord(ARMASpec);
  InitialiseARMARecord(ARMAEst);

  repeat
    ClrScr;
    Writeln;
    Writeln;
    Writeln('    OVERSAMPLE.PAS: Investigating Band-Limited ARMA Processes');
    Writeln;
    Writeln('         1. Page Parameters');
    Writeln;
    Writeln('         2. Specify an ARMA Model');
    Writeln;
    Writeln('         3. Plot the Autocovariance Function');
    Writeln;
    Writeln('         4. Plot the Spectral Density');
    Writeln;
    Writeln('         5. Plot the Pole-Zero Diagram');
    Writeln;
    Writeln('         6. Determine the Misspecified Parameters');
    Writeln;
    Writeln('         7. SAVE and EXIT');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 7, theMenuItem);

    case theMenuItem of
      1: 
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);{in UTILS}
      2: 
        SpecifyTheModel(ARMASpec);
      3: 
        PlotTheoreticalACF(ARMASpec, SCREENBOX, POSTBOX, spectrumVec, spectrumBOX); {in ORGANISE}
      4: 
        PlotTheSpectrum(ARMASpec, SCREENBOX, POSTBOX, spectrumVec, spectrumBOX); {in ORGANISE}
      5: 
        PoleZeroDiagram(ARMASpec, ARMAEst, SCREENBOX, POSTBOX);
      6: 
        MisspecifiedModel(ARMASpec, ARMAEst, SCREENBOX, POSTBOX, spectrumVec, spectrumBOX);
      7: 
        SaveAndExit(ARMASpec, ARMAEst);
    end;

  until allsDone;

  RevertText;

end.  {OVERSAMPLE Main Unit}

    {________________________________________________}

   {Compile the Units in the Following Order:}

   {1. GLOBALS}
   {2. MATHS}
   {3. UTILS}
   {4. SCREEN}
   {5. POST}
   {6. ROOTS}
   {7. SPECTRAL}
   {8. ACQUIRE}
   {9. AUTOREG}
   {10. DATA}
   {11. ORGANISE}
   {12. MAIN}

