unit MODELEST; {Unit 9 of OVERSAMPLE.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, ACOVARS;


  procedure YuleWalker (p, q: integer;
                  gamma: vector;
                  var alpha: vector;
                  var varEpsilon: real);

  procedure Minit (var mu: vector;
                  var varEpsilon: real;
                  gamma: vector;
                  q: integer);

  procedure ARMAparameters (p, q: integer;
                  gamma: vector;
                  var alpha, mu: vector;
                  var varEpsilon: real);

implementation

{_________________________________________________}

  procedure YuleWalker (p, q: integer;
                  gamma: vector;
                  var alpha: vector;
                  var varEpsilon: real);

    var
      a: matrix;
      b: vector;
      i, j, k: integer;

  begin {Yule-Walker}
    for i := 1 to p do
      begin {i}
        b[i] := -gamma[q + i];
        for j := 1 to p do
          begin {j}
            k := abs(q + i - j);
            a[i, j] := gamma[k];
          end{j}
      end; {i}

    LUsolve(1, p, a, alpha, b);
    alpha[0] := 1;
    varEpsilon := 0;
    for i := 0 to p do
      varEpsilon := varEpsilon + alpha[i] * gamma[i];

  end; {Yule-Walker}

{_________________________________________________}

  procedure Minit (var mu: vector;
                  var varEpsilon: real;
                  gamma: vector;
                  q: integer);

    var
      d: matrix;
      delta, f: vector;
      i, j, iterations: integer;
      convergence: boolean;

{------------------------------------------}

    function CheckDelta (q: integer;
                    var delta, mu: vector): boolean;

      var
        i: integer;
        muNorm, deltaNorm: real;

    begin
      muNorm := 0.0;
      deltaNorm := 0.0;
      for i := 0 to q do
        begin
          muNorm := muNorm + sqr(mu[i]);
          deltaNorm := deltaNorm + sqr(delta[i])
        end;
      if (deltaNorm / muNorm) > 0.00001 then
        CheckDelta := false
      else
        CheckDelta := true;

    end; {CheckDelta}

{------------------------------------------}

  begin {procedure Minit}

{initialise the vector mu }
    for i := 0 to q do
      mu[i] := 0.0;
    mu[0] := sqrt(gamma[0]);
    convergence := false;
    iterations := 0;

    while (convergence = false) and (iterations < 10) do
      begin

{Form the matrix of derivatives}
        for i := 0 to q do
          for j := 0 to q do
            begin
              d[i, j] := 0.0;
              if (j - i) >= 0 then
                d[i, j] := mu[j - i];
              if (i + j) <= q then
                d[i, j] := d[i, j] + mu[i + j];
            end;

{Find the function value}
        for j := 0 to q do
          begin
            f[j] := gamma[j];
            for i := 0 to q - j do
              f[j] := f[j] - mu[i] * mu[i + j]
          end;

{Find the updating vector}
        LUsolve(0, q + 1, d, delta, f);

{Update the value of mu}
        for i := 0 to q do
          mu[i] := mu[i] + delta[i];
        iterations := iterations + 1;

{Check for convergence}
        convergence := CheckDelta(q, delta, mu);
      end; {while}
    Writeln('    The number of iterations was ', iterations : 3);

{Renomalise the results}
    varEpsilon := 1.0;
    for i := 1 to q do
      begin {i}
        mu[i] := mu[i] / mu[0];
        varEpsilon := varEpsilon + mu[i] * mu[i];
      end; {i}
    mu[0] := 1;
    varEpsilon := gamma[0] / varEpsilon;

  end; {Minit}

{_________________________________________________}

  procedure ARMAparameters (p, q: integer;
                  gamma: vector;
                  var alpha, mu: vector;
                  var varEpsilon: real);

    var
      t, i, j, k: integer;
      temp: real;
      psi: vector;

  begin {ARMAparameters}

    YuleWalker(p, q, gamma, alpha, temp);

    for t := 0 to q do
      begin {t}
        psi[t] := 0.0;
        for i := 0 to p do
          for j := 0 to p do
            begin {i, j}
              k := abs(t + j - i);
              psi[t] := psi[t] + alpha[i] * alpha[j] * gamma[k];
            end; {i, j}
      end;{t}

    if q > 0 then
      Minit(mu, varEpsilon, psi, q)
    else
      varEpsilon := temp;

  end;{ARMAparameters}

{_________________________________________________}

end. {MODELEST: Unit 9 of OVERSAMPLE.PAS}

