unit ORGANISE; {Unit 10 of OVERSAMPLE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, ACQUIRE, SCREEN, POST, SPECTRAL, ACOVARS, MODELEST, ROOTS, Graph, Dos, Crt;

  procedure SpecifyTheModel (var ARMASpec: ARMARecord);

  procedure PlotTheSpectrum (var ARMASpec: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure PlotTheoreticalACF (var ARMASpec: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure MisspecEstimates (var ARMASpec, ARMAEst: ARMARecord;
                  var spectrumVec: longVector);

  procedure PlotTheSampledACF (var ARMASpec, ARMAEst: ARMARecord;
                  var spectrumVec: longVector);

  procedure OverlaidSpectra (var ARMASpec, ARMAEst: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longvector);

  procedure PoleZeroDiagram (var ARMASpec, ARMAEst: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX);

  procedure MisspecifiedModel (var ARMASpec, ARMAEst: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure SaveARMAParameters (var ARMASpec, ARMAEst: ARMARecord);

  procedure SaveAndExit (var ARMASpec, ARMAEst: ARMARecord);

implementation

{________________________________________________}

  procedure SpecifyTheModel (var ARMASpec: ARMARecord);

    var
      i, lags: integer;

  begin
    InitialiseARMARecord(ARMASpec);
    InitialiseARMARecord(ARMAEst);

    ARMAacquisition(ARMASpec, stability); {inACQUIRE}
    lags := ARMASpec.p + ARMASpec.q + 1;

    with ARMASpec do {in ACOVARS}
      begin
        ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);

        ClrScr;
        Writeln;
        Writeln('    THE COEFFICIENTS OF THE UNDERLYING MODEL');
        Writeln;
        Writeln('    The white-noise forcing function has unit variance');
        Writeln;
        Writeln('    Denominator Coefficients');
        Writeln;
        for i := 0 to ARMASpec.p do
          Writeln('    alpha(', i : 1, ') = ', alpha[i] : 8 : 4);

        Writeln;
        Writeln('    Numerator Coefficients');
        Writeln;
        for i := 0 to ARMASpec.q do
          Writeln('    mu(', i : 1, ') = ', mu[i] : 8 : 4);

        Writeln;
        Writeln('    The Leading Autocovariances');
        Writeln;
        for i := 0 to lags do
          Writeln('    gamma(', i : 1, ') = ', gamma[i] : 8 : 4);
        Writeln;
        Writeln('    Press <RETURN> to continue');
        Readln;
{Scale varepsilon to ensure that the spectrum has unit area}
{varepsilon := varepsilon / gamma[0]; NO}
        parametersFound := true;
      end; {with ARMASpec}

  end; {ARMASpec: ARMARecord}

{________________________________________________}

  procedure PlotTheSpectrum (var ARMASpec: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);
    var
      legend, postFileName: string;
       noiseSpectrum: real;
      lags: integer;
      F: text;

  begin
  
    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if (ARMASpec.parametersFound = false) then
      begin
        ClrScr;
        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You have not specified an ARMA model and you have not estimated');
        Writeln('    any parameters. You are asked to specify an ARMA model of which');
        Writeln('    the spectral density function will be displayed.');
        Writeln;
        Writeln(' ' : 59, '<RETURN>');
        Readln;
        ARMAacquisition(ARMASpec, stability);

        with ARMASpec do
          begin
            lags := p + q + 1;
            ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);
{varEpsilon has been set to unity wthin ARMAacquisition}
{Scale varEpsilon to ensure that the spectrum has unit area}
{varEpsilon := varEpsilon / gamma[0];NO}
          end;
      end; {parametersFound  = false}

    MakeSpectrumVec(ARMASpec, SCREENBOX, spectrumVec, spectrumBOX); {in SPECTRAL}

{Plotting}
    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrameLite(spectrumBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);

    ScreenPlotMod(spectrumBOX, SCREENBOX, spectrumVec);
    legend := 'The parametric spectrum of a model specified by the user';

{if ARMASpec.contaminated then}
{begin}
{noiseSpectrum := ARMASpec.noiseVar / twoPi;}
{PlotAConstant(spectrumBOX, SCREENBOX, noiseSpectrum);}
{legend := Concat(legend, ' supported on the spectrum of a white-noise contamination');}
{end;}

    ScreenCaption(legend);

{Do the PostScript}

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {Make a PostScript File}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {PostfileName}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            PostFrameLite(F, spectrumBOX, POSTBOX);
            PostZeroPiAxis(F, POSTBOX);
            PostPlotMod(F, spectrumBOX, POSTBOX, spectrumVec);
            if ARMASpec.contaminated then
              PostConstant(F, spectrumBOX, POSTBOX, noiseSpectrum);
            PostFooter(F, POSTBOX, legend, TexFormat);
            Close(F);
          end; {postFileName}
      end; {Make a PostScript File}
    ARMASpec.spectrumFound := true;

  end; {PlotTheSpectrum}

{_________________________________________________}

  procedure PlotTheoreticalACF (var ARMASpec: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

    var
      correlVec: longVector;
      correlBOX: rBOX;
      lags, Ncap: integer;
      autoMin, omegaFactor: real;
      continue: boolean;
      legend: string;

  begin
    legend := ' ';

    if (ARMASpec.parametersFound = false) then
      begin
        SysBeep(3);
        ARMAacquisition(ARMASpec, stability);
        lags := ARMASpec.p + ARMASpec.q + 1;
        with ARMASpec do {it looks as if this one is redundant}
          ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q); {where is lags coming feodm}
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    ClrScr;
    Writeln;
    Writeln('    THEORETICAL AUTOCORRELATION FUNCTION');
    Writeln;
    Writeln('    How many autocorrelations do you wish to plot?');
    GetBoundedInteger(10, 50, lags);

    with ARMASpec do
      begin {ARMASpec}
        ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);
        autocovariancesFound := true;
        ARMACorrelations(lags, rho, gamma, autoMin);

        SCREENBOX.X2 := SCREENBOX.X1 + 256;
        CorrelBOX.NOPUNCT := 256;
        Ncap := 256;
        correlBOX.xMax := lags + 1;
        correlBOX.xMin := 0.0;
        correlBOX.yMax := 1.0;
        correlBOX.yMin := autoMin;

        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        ScreenFrame(CorrelBOX, SCREENBOX);
        SpikePlot(lags, CorrelBOX, SCREENBOX, rho);

        legend := 'The autocorrelation function of the specified process';
        legend := Concat(legend, ' determined from the ARMA parameters');
        ScreenCaption(legend);

        MakeSpectrumVec(ARMASpec, SCREENBOX, spectrumVec, spectrumBOX);

        CallForReturn;
        Readln;
        CloseGraph;
        RestoreCrtMode;
        ClrScr;

        {GetGraphics(graphicsOn, SCREENBOX, currentMode); }
        {ScreenFrameLite(spectrumBOX, SCREENBOX);}
        {ZeroPiAxis(SCREENBOX);}
        {ScreenPlotMod(spectrumBOX, SCREENBOX, SpectrumVec);}
        {speclegend := 'The parametric spectrum of a model specified by the user';}
        {ScreenCaption(speclegend); }

        {CallForReturn;}
        {Readln;}
        {CloseGraph;}
        {RestoreCrtMode;}
        {ClrScr;}

        Writeln;
        Writeln('    THE CONTINUOUS AUTOCORRELATION FUNCTION');
        Writeln;
        YesOrNo('    Do you wish to plot the continuous autocorrelation function', Continue);

        if Continue then
          begin {Continuous Plot}
            Writeln('    This may take time. Please be patient.');

            omegaFactor := lags + 1;
            omegaFactor := omegaFactor / Ncap;

            SlowInverseCosine(omegaFactor, CorrelVec, SpectrumVec, Ncap, lags);
            MakeDataBox(correlBOX, correlVec);
            CorrelBOX.xMax := lags + 1;
            CorrelBOX.xMin := 0.0;

            CorrelPlot(plotOne, ARMASpec, SCREENBOX, POSTBOX, correlBOX, correlVec, correlVec, lags);
          end;{Continuous Plot}
{CorrelPlot will plot the rho vector via SpikePlot}

      end {ARMASpec}
  end; {PlotTheoreticalACF}

{_________________________________________________}

  procedure MisspecEstimates (var ARMASpec, ARMAEst: ARMARecord;
                  var spectrumVec: longVector);

{ARMASpec SpectrumVec  has already been determined wthin the calling routine}
{MisspecifiedModel}

    var
      lagOrder, Ncap, i: integer;
      contaminate: boolean;
      varFraction: real;

  begin
    Ncap := 256;

    with ARMAEst do
      begin
        ClrScr;
        Writeln;
        Writeln;
        Writeln('  SPECIFY THE SAMPLING RATE');
        Writeln;
        Writeln('    Specify the rate of sampling in terms of observations per unit of time.');
        Writeln('    The sampling rate appropriate to the specified model is one observation');
        Writeln('    per unit of time ');
        Writeln;
        Writeln('    Choose a value for the sample rate in the interval (1 , 8) ');
        GetBoundedInteger(1, 8, ARMASpec.sampleRate); {ARMAEst.sampleRate = 1, always}
        ARMASpec.omega2Deg := 180.0 / ARMASpec.sampleRate;

        Writeln;
        Writeln('    Specify the order of the autoregressive moving-average  model to');
        Writeln('    be estimated from ordinates sampled from the autocovariance');
        Writeln('    function of the continuous time model.');
        Writeln;
        Writeln('    What is the order p of auroregressive operator ?');
        GetBoundedInteger(0, 8, p);
        Writeln;
        Writeln('    What is the order q of moving-average operator ?');
        GetBoundedInteger(0, 8, q);
        ordersSpecified := true;

        ARMASpec.contaminated := false;
        YesOrNo('    Do you wish to add a white-noise contamination to the process', contaminate);

{There are two ways of incorporating the white noise contamination in the model.}
{First, we might add it to the spectrum by supporting the latter on a flat noise}
{spectrum. When this is transformed back to the frequency domain, then only the}
{variance should be altered. The alternative is to use the uncontaminated spectrum}
{to generate the autocovariances and then to add the noise elements to the variance}
{and the adjacent autocovariances. Here we adopt the second procedure. We assume}
{that this is a discrete-time noise that is applied directly to the sampled elements}

{Now we are in a position to find the autocovariances and to estimate the parameters}

        lagOrder := p + q + 1; {ARMAEST}
        OverSampledCovars(rho, spectrumVec, Ncap, lagOrder, ARMASpec.sampleRate);

        if contaminate then
          begin {contaminate}
            Writeln('    Specify the contamination as a fraction the variance of the process');
            ReadBoundedNumber(0.0, 1.0, varFraction);
            ARMASpec.noiseVar := varFraction * ARMASpec.gamma[0];
            ARMASpec.contaminated := true;
            rho[0] := rho[0] * (1.0 + varFraction);
          end; {contaminate}
{ARMASpec.contaminated := false; will be imposed when we exit MisspecifiedModel }

        ARMAparameters(p, q, rho, alpha, mu, varEpsilon); {Estimates we are within ARMAEST}
        if q = 0 then
          mu[0] := 1.0;
      end; {with ARMAEst}
    ARMAEst.parametersFound := true;

{++++++++++++++++++++++++++++++++++++++++++++++}

 {Print out the acquired coefficients}
    Writeln;
    Writeln('    THE COEFFICIENTS OF THE SPECIFIED MODEL');
    Writeln;
    Writeln('    The variance of the white-noise forcing function: ', ARMASpec.varEpsilon : 6 : 4);
    Writeln;
    Writeln('    The variance of the specified process: ', ARMASpec.gamma[0] : 6 : 4);
    Writeln;
    if ARMASpec.contaminated then
      Writeln('    The variance of the white-noise contamination: ', ARMASpec.noiseVar : 6 : 4);
    Writeln;

    Writeln('    Denominator Coefficients');
    Writeln;
    for i := 0 to ARMASpec.p do
      Writeln('    alpha(', i : 1, ') = ', ARMASpec.alpha[i] : 8 : 4);

    Writeln;
    Writeln('    Numerator Coefficients');
    Writeln;
    for i := 0 to ARMASpec.q do
      Writeln('    mu(', i : 1, ') = ', ARMASpec.mu[i] : 8 : 4);

    Writeln;
    Writeln('    Press <RETURN> to continue');
    Readln;
    ClrScr;

  end; {MisspecEstimates}

{_________________________________________________}

  procedure PlotTheSampledACF (var ARMASpec, ARMAEst: ARMARecord;
                  var spectrumVec: longVector);

    var
      TrueCorrVec, EstCorrVec: longVector;
      CorrelBOX, secondBox: rBOX;
      lags, Ncap: integer;
      omegaFactor: real;
   
  begin
    NCap := 256;

    if ARMASpec.spectrumFound = false then
      MakeSpectrumVec(ARMASpec, SCREENBOX, spectrumVec, spectrumBOX);

    if (ARMAEst.ordersSpecified = false) then
      MisspecEstimates(ARMASpec, ARMAEst, spectrumVec);

    ClrScr;
    Writeln;
    Writeln('    PLOT THE AUTOCORRELATION FUNCTION');
    Writeln;
    Writeln('    How many autocorrelations do you wish to plot?');
    GetBoundedInteger(10, 50, lags);
    Writeln('    Plotting the continuous autocovariance functions can take time');

    omegaFactor := lags + 1;
    omegaFactor := omegaFactor / (ARMASpec.sampleRate * Ncap);

    SlowInverseCosine(omegaFactor, TrueCorrVec, SpectrumVec, Ncap, lags);
    CorrelBOX.NOPUNCT := 256;
    MakeDataBox(CorrelBOX, TrueCorrVec);
    CorrelBOX.xMax := lags + 1;
    CorrelBOX.xMin := 0.0;

{Pensize(2, 2);}
    {CorrelPlot(plotOne, ARMASpec, SCREENBOX, POSTBOX, CorrelBOX, TrueCorrVec, TrueCorrVec, lags);}
{PenNormal;}

    MakeSpectrumVec(ARMAEst, SCREENBOX, spectrumVec, spectrumBOX);
    ARMASpec.spectrumFound := false;
    ARMAEst.spectrumFound := true;
    omegaFactor := (lags + 1) / Ncap;
    SlowInverseCosine(omegaFactor, EstCorrVec, SpectrumVec, Ncap, lags);
    ARMAEst.autocovariancesFound := true;
    secondBOX := correlBOX;
    MakeDataBox(secondBOX, EstCorrVec);
    correlBOX.yMin := Rmin(correlBOX.yMin, secondBOX.yMin);
    CorrelPlot(plotTwo, ARMASpec, SCREENBOX, POSTBOX, correlBOX, TrueCorrVec, EstCorrVec, lags);

  end; {PlotTheSampledACF}

{_________________________________________________}

  procedure OverlaidSpectra (var ARMASpec, ARMAEst: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var SpectrumVec: longvector);

    var
      LimSpecVec, EstSpecVec: longVector;
      spectrumBOX, bandSpecBOX: rBOX;
      legend, postFileName: string;
      F: text;
      lags: integer;

  begin

    if (ARMAEst.ordersSpecified = false) then
      MisspecEstimates(ARMASpec, ARMAEst, spectrumVec);

    with ARMASpec do
      begin
        lags := 2; {we are only looking for the variance}
        varEpsilon := 1;
        ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);
        if contaminated then
          varEpsilon := varEpsilon / (gamma[0] + noiseVar)
        else
          varEpsilon := varEpsilon / gamma[0];
      end;
    with ARMAEst do
      begin
        lags := 2;
        varEpsilon := 1;
        ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);
        varEpsilon := varEpsilon / gamma[0];
      end;

    MakeSpectrumVec(ARMAEst, SCREENBOX, EstSpecVec, spectrumBOX);
    MakeScreenBandSpec(ARMASpec, SCREENBOX, LimSpecVec, bandSpecBOX);
    if bandSpecBOX.yMax > spectrumBOX.yMax then
      spectrumBOX.yMax := bandSpecBOX.yMax;

{Plotting}

    GetGraphics(graphicsOn, SCREENBOX, currentMode);
    ScreenFrameLite(spectrumBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);
    SetColor(red);
    ScreenPlotMod(spectrumBOX, SCREENBOX, EstSpecVec);
    SetColor(yellow);

    legend := 'The parametric spectrum of an estimated model (red line)';
    legend := Concat(legend, ' and of the underlying process');
    if ARMASpec.contaminated then
      begin
        PlotLittoralSpectrum(ARMASpec, spectrumBOX, SCREENBOX);
        legend := Concat(legend, ' supported on the spectrum of a white-noise contamination');
      end;

    ScreenCaption(legend);

    {SetLineStyle(solidLn, 0, thickWidth);}
    ScreenBandPlot(ARMASpec, spectrumBOX, SCREENBOX, LimSpecVec);
    {SetLineStyle(solidLn, 0, normWidth);}
    if (ARMASpec.sampleRate > 1) then
      LimitLine(SCREENBOX, ARMASpec);

    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then {make a PostScript File}
      begin {PSPict}
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {PostfileName}
            Assign(F, postFileName);
            Rewrite(F);
            PostHeader(F, POSTBOX, TeXFormat);
            PostFrameLite(F, spectrumBOX, POSTBOX);
            PostZeroPiAxis(F, POSTBOX);
            MakePostBandSpec(ARMASpec, POSTBOX, LimSpecVec, spectrumBOX);
            Writeln(F, '1.0 setlinewidth');
            PostBandPlot(F, ARMASpec, spectrumBOX, POSTBOX, LimSpecVec);
{Writeln(F, '0.5 setlinewidth');}
            PostLimitLine(F, POSTBOX, ARMASpec);
            PostPlotMod(F, spectrumBOX, POSTBOX, EstSpecVec);
            if ARMASpec.contaminated then
              PostLittoralSpectrum(F, ARMASpec, spectrumBOX, POSTBOX);
            legend := ' ';
            PostFooter(F, POSTBOX, legend, TexFormat);
            Close(F);
          end; {PostFileName}
      end; {PSPict}

    ARMAEst.varEpsilon := 1.0;
    ARMASpec.varEpsilon := 1.0;

  end; {OverlaidSpectra}

{_________________________________________________}

  procedure PoleZeroDiagram (var ARMASpec, ARMAEst: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX);

    var
      ARGANDBOX: IBOX;
      duplex: boolean;
      leftShift, lags: integer;
      postFileName, legend: string;
      F: text;

  begin
    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if (ARMASpec.parametersFound = false) then
      begin
        ClrScr;
        Writeln;
        Writeln('    PLOT THE POLE-ZERO DIAGRAM');
        Writeln;
        Writeln('    You have not specified an ARMA model and you have not estimated');
        Writeln('    any parameters. You are asked to specify an ARMA model of which');
        Writeln('    the ploe-zero diagram will be displayed.');
        Writeln;
        Writeln(' ' : 59, '<RETURN>');
        Readln;
        ARMAacquisition(ARMASpec, stability);

        with ARMASpec do
          begin
            lags := p + q + 1;
            ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);
{varEpsilon has been set to unity wthin ARMAacquisition}
{Scale varEpsilon to ensure that the spectrum has unit area}
            varEpsilon := varEpsilon / gamma[0];
          end;
      end; {parametersFound  = false}


    if (ARMAEst.parametersFound) and (ARMAEst.rootsFound = false) then
      FindPolesAndZeros(ARMAEst);

    duplex := false;
    if (ARMAEst.parametersFound = false) then
      begin
        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        ARGANDBOX := SCREENBOX;
        ScreenAargand(ARGANDBOX, ARMASpec);
        legend := 'The Pole-Zero for the Specified Process';
        ScreenCaption(legend);
      end
    else if (ARMAEst.parametersFound) then
      begin
        duplex := true;
        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        ARGANDBOX := SCREENBOX;
        ARGANDBOX.XC := ARGANDBOX.XC - 120;
        ScreenAargand(ARGANDBOX, ARMASpec);
        leftShift := TextWidth('Specified Process') div 2;
        MoveTo(ARGANDBOX.XC - leftShift, ARGANDBOX.YC + 100);
        OutText('Specified Process');
        ARGANDBOX.XC := ARGANDBOX.XC + 240;
        ScreenAargand(ARGANDBOX, ARMAEst);
        leftShift := TextWidth('Estimated Model') div 2;
        MoveTo(ARGANDBOX.XC - leftShift, ARGANDBOX.YC + 100);
        OutText('Estimated Model');
        legend := 'The Pole-Zero Diagrams for the Specified  Process (left) and for the Estimated Process';
        ScreenCaption(legend);
      end;

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin
        Write('    Name the PostScript File = ');
        Readln(postFileName);
        if postFileName <> '' then
          begin {PostfileName}
            Assign(F, postFileName);
            Rewrite(F);
            PostAargandHeader(F, TeXFormat);

            if (duplex = false) then
              begin {false}
                Writeln(F, '/units {55 mul} def');
                if TeXFormat then
                  Writeln(F, '165 -100 translate')
                else
                  Writeln(F, '250 650 translate');
                PostAargand(F, duplex, ARMASpec);
              end; {false}

            if duplex = true then
              begin {true}
                Writeln(F, '/units {40 mul} def');
                if TeXFormat then
                  Writeln(F, '75 -100 translate')
                else
                  Writeln(F, '160  650 translate');
                PostAargand(F, duplex, ARMASpec);
                if TeXFormat then
                  Writeln(F, 175 : 3, 0 : 3, '  translate')
                else
                  Writeln(F, '175  0 translate');
                PostAargand(F, duplex, ARMAEst);
              end; {true}

            PostAargandFooter(F, TeXFormat);
            if TeXFormat then
{PoleZeroLegend(F, duplex, AREstim, ARMASpec);}
              Close(F);
          end; {postFileName}
      end;{PSPict}

  end; {PoleZeroDiagram}

{________________________________________________}

  procedure MisspecifiedModel (var ARMASpec, ARMAEst: ARMARecord;
                  var SCREENBOX, POSTBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);
    var
      lags, theMenuItem: integer;

  begin

    if (ARMASpec.parametersFound = false) then
      begin
        SysBeep(3);
        ClrScr;
        Writeln;
        Writeln('    SPECIFY AN ARMA MODEL FOR THE UNDERLYING PROCESS');
        Writeln;
        Writeln('    You have  not yet  specified the  ARMA model which will generate');
        Writeln('    the continuous autocovariance function from which values will be');
        Writeln('    sampled that give rise to an estimated model.');
        Writeln;
        Writeln(' ' : 60, '<RETURN>');
        Readln;
        ARMAacquisition(ARMASpec, stability);
        with ARMASpec do
          begin
            lags := p + q + 1;
            ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);
{varEpsilon := varEpsilon / gamma[0]; NOT HERE}
          end;
      end;

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if (ARMASpec.spectrumFound = false) then
      PlotTheSpectrum(ARMASpec, SCREENBOX, POSTBOX, spectrumVec, spectrumBOX);
{MakeSpectrumVec(ARMASpec, SCREENBOX, spectrumVec, spectrumBOX);}

{We might replace this by MakeSpectrumVec.  We need to determine the }
{spectrum in order to pass it to the routine MisspecEstimates, where it }
{will be used for determining the oversampled autocovariances}

    ClrScr;
    Writeln;
    Writeln;
    Writeln;
    Writeln('    DETERMINE THE MISSPECIFIED PARAMETERS');
    Writeln;
    Writeln('    In this section, we infer the parameters  of  a discrete-time ARMA model');
    Writeln('    from ordinates that  have been  sampled from the autocovariance function');
    Writeln('    of  an already  specified continuous-time ARMA  model that is  supported');
    Writeln('    on the Nyquist interval.');
    Writeln;
    Writeln('    The correct  model will  be recovered  from  the  autocovariance function');
    Writeln('    only if the orders are correctly specified and if the function is sampled');
    Writeln('    at the  appropriate rate. The program  allows the orders to be  specified');
    Writeln('    incorrectly and it allows the rate of sampling to be excessive.');
    Writeln;
    Writeln('    Sampling the  continuous-time  process at an excessive rate will generate');
    Writeln('    a discrete-time process that is supported on a subinterval of the Nyquist');
    Writeln('    range. A  model fitted to the  resulting autocovariances may deliver some');
    Writeln('    severely misleading estimates.');
    Writeln;
    Writeln(' ' : 70, '<RETURN>');
    Readln;

    repeat
      ClrScr;
      Writeln;
      Writeln;
      Writeln('    DETERMINE THE MISSPECIFIED PARAMETERS');
      Writeln;
      Writeln('         1. Specify the ARMA Orders and the Sampling Rate');
      Writeln;
      Writeln('         2. Plot the Autocovariance Function and its Sampled Ordinates');
      Writeln;
      Writeln('         3. Plot the Spectral Density of the Misspecified Model');
      Writeln;
      Writeln('         4. Plot the Poles and Zeros of the Misspecified Model');
      Writeln;
      Writeln('         5. RETURN to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press <RETURN>');
      GetBoundedInteger(1, 5, theMenuItem);

      case theMenuItem of
        1: 
          MisspecEstimates(ARMASpec, ARMAEst, spectrumVec);
        2: 
          PlotTheSampledACF(ARMASpec, ARMAEst, spectrumVec);
        3: 
          OverlaidSpectra(ARMASpec, ARMAEst, SCREENBOX, POSTBOX, spectrumVec);
        4: 
          PoleZeroDiagram(ARMASpec, ARMAEst, SCREENBOX, POSTBOX);
      end; {cases}

    until (theMenuItem = 5);

    ARMASpec.contaminated := false;
    ARMASpec.rho[0] := 1.0;

  end; {MisspecifiedModel}

{_________________________________________________}

  procedure SaveARMAParameters (var ARMASpec, ARMAEst: ARMARecord);

    var
      j, lags: integer;
      effectivePoles: polarVector;
      CartesianPoles: complexVector;
      resultsFileName: string;
      F: Text;
   begin
    ClrScr;

    Write('    Name the File for the Results = ');
    Readln(resultsFileName);
    if resultsFileName <> '' then
      begin {Save the Results}
        Assign(F, resultsFileName);
        Rewrite(F);
        Writeln(F);

        with ARMASpec do
          begin {ARMASpec}

            Writeln(F, '    THE PARAMETERS THE SPECIFIED MODEL');

            Writeln(F);
            Writeln(F, '      varEpsilon = ', varEpsilon : 6 : 4);
            if contaminated then
              begin
                Writeln(F);
                Writeln(F, '      noise Variance = ', noiseVar : 6 : 4);
              end;
            Writeln(F);
            Writeln(F, '    The Autocovariances');
            Writeln(F);
            for j := 0 to p + q + 1 do
              Writeln(F, '      gamma[', j : 1, '] =', gamma[j] : 6 : 4);

            if (p > 0) then
              begin {p > 0}
                Writeln(F);
                Writeln(F, '    The Autoregressive Parameters');
                Writeln(F);
                for j := 0 to p do
                  Writeln(F, '      alpha[', j : 1, '] =', alpha[j] : 6 : 4);

                Writeln(F);
                Writeln(F, '    The Autoregressive Roots');
                Writeln(F);
                for j := 1 to p do
                  Writeln(F, '      pole[', j : 1, '].re = ', poles[j].re : 5 : 5, '   pole[', j : 1, '].im = ', poles[j].im : 5 : 5);
                Writeln(F);

                for j := 1 to p do
                  begin {j}
                    Writeln(F, '      pole(', j : 1, ').modu = ', polarPoles[j].modu : 8 : 4);
                    Writeln(F, '      pole(', j : 1, ').argu = ', polarPoles[j].argu : 8 : 4);
                    Writeln(F);
                  end; {j}

                if (sampleRate > 1) then
                  begin {sampleRate}
                    Writeln(F, '    THE EFFECTIVE AUTOREGRESSIVE POLES');
                    Writeln(F);
                    Writeln(F, '      The sample rate is ', sampleRate : 1);

                    EffectivePolarPoles(ARMASPec, effectivePoles);
                    EffectiveCartesianPoles(ARMASPec, CartesianPoles);

                    Writeln(F);
                    for j := 1 to p do
                      Writeln(F, '      pole[', j : 1, '].re = ', CartesianPoles[j].re : 5 : 5, '   pole[', j : 1, '].im = ', CartesianPoles[j].im : 5 : 5);
                    Writeln(F);

                    for j := 1 to p do
                      begin {j}
                        Writeln(F, '      pole(', j : 1, ').modu = ', effectivePoles[j].modu : 8 : 4);
                        Writeln(F, '      pole(', j : 1, ').argu = ', effectivePoles[j].argu : 8 : 4);
                        Writeln(F);
                      end; {j}
                  end; {sampleRate}
              end; {p>0}

            if (q > 0) then
              begin {q > 0}

                Writeln(F);
                Writeln(F, '    The Moving-Average Parameters');
                Writeln(F);
                for j := 0 to q do
                  Writeln(F, '      mu[', j : 1, '] =', mu[j] : 6 : 4);

                Writeln(F);
                Writeln(F, '    The Moving-Average Roots');
                Writeln(F);
                for j := 1 to p do
                  Writeln(F, '      zero[', j : 1, '].re = ', zeros[j].re : 5 : 5, '   zero[', j : 1, '].im = ', zeros[j].im : 5 : 5);
                Writeln(F);

                for j := 1 to p do
                  begin {j}
                    Writeln(F, '      zero(', j : 1, ').modu = ', polarZeros[j].modu : 8 : 4);
                    Writeln(F, '      zero(', j : 1, ').argu = ', polarZeros[j].argu : 8 : 4);
                    Writeln(F);
                  end; {j}

              end; {q > 0}
          end; {ARMASpec}

        Writeln(F);

        if ARMAEst.parametersFound then
          with ARMAEst do
            begin {ARMAEst}

              if (rootsFound = false) then
                FindPolesAndZeros(ARMAEst);
              lags := p + q + 1;
              if (autocovariancesFound = false) then
                ARMACovariances(alpha, mu, gamma, varEpsilon, lags, p, q);

              Writeln(F, '    THE PARAMETERS THE ESTIMATED MODEL');
              Writeln(F);
              Writeln(F);
              Writeln(F, '      varEpsilon = ', varEpsilon : 6 : 4);
              if contaminated then
                begin
                  Writeln(F);
                  Writeln(F, '      noise Variance = ', noiseVar : 6 : 4);
                end;
              Writeln(F);
              Writeln(F, '    The Autocovariances');
              Writeln(F);
              for j := 0 to p + q + 1 do
                begin {j}
                  Writeln(F, '      gamma[', j : 1, '] =', gamma[j] : 6 : 4);
                end; {j}

              if (p > 0) then
                begin {p > 0}
                  Writeln(F);
                  Writeln(F, '    The Autoregressive Parameters');
                  Writeln(F);
                  for j := 0 to p do
                    begin {j}
                      Writeln(F, '      alpha[', j : 1, '] =', alpha[j] : 6 : 4);
                    end; {j}

                  Writeln(F);
                  Writeln(F, '    The Autoregressive Roots');
                  Writeln(F);
                  for j := 1 to p do
                    Writeln(F, '      pole[', j : 1, '].re = ', poles[j].re : 5 : 5, '   pole[', j : 1, '].im = ', poles[j].im : 5 : 5);
                  Writeln(F);
                  for j := 1 to p do
                    begin {j}
                      Writeln(F, '      pole(', j : 1, ').modu = ', polarPoles[j].modu : 8 : 4);
                      Writeln(F, '      pole(', j : 1, ').argu = ', polarPoles[j].argu : 8 : 4);
                      Writeln(F);
                    end; {j}
                end; {p>0}

              if (q > 0) then
                begin {q > 0}

                  Writeln(F);
                  Writeln(F, '    The Moving-Average Parameters');
                  Writeln(F);
                  for j := 0 to q do
                    begin {j}
                      Writeln(F, '      mu[', j : 1, '] =', mu[j] : 6 : 4);
                    end; {j}
                  Writeln(F);
                  Writeln(F, '    The Moving-Average Roots');
                  Writeln(F);
                  for j := 1 to p do
                    Writeln(F, '      zero[', j : 1, '].re = ', zeros[j].re : 5 : 5, '   zero[', j : 1, '].im = ', zeros[j].im : 5 : 5);
                  Writeln(F);
                  for j := 1 to p do
                    begin {j}
                      Writeln(F, '      zero(', j : 1, ').modu = ', polarZeros[j].modu : 8 : 4);
                      Writeln(F, '      zero(', j : 1, ').argu = ', polarZeros[j].argu : 8 : 4);
                      Writeln(F);
                    end; {j}

                end; {q > 0}
            end; {ARMAEst}

        Writeln(F);
        Close(F);

      end;{Save the Results}

  end; {SaveARMAParameters}

{_________________________________________________}

  procedure SaveAndExit (var ARMASpec, ARMAEst: ARMARecord);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;

      Writeln;
      Writeln('    SAVE AND EXIT');
      Writeln;
      Writeln('      1. SAVE ARMA Parameters');
      Writeln;
      Writeln('      2. RETURN to the Main Menu');
      Writeln;
      Writeln('      3. EXIT the Program');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 3, theMenuItem);

      case theMenuItem of
        1: 
          SaveARMAParameters(ARMASpec, ARMAEst);
        2: 
          ;{RETURN to the Main Menu}
        3: 
          allsDone := true; {EXIT the Program}
      end;{case}

    until theMenuItem in [1..3];

  end;{SaveAndExit}

{________________________________________________}

end. {ORGANISE: Unit 10 of OVERSAMPLE.PAS}

