unit POST; {Unit 6 of OVERSAMPLE.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, ROOTS;

  procedure PostHeader (var F: text;
                  var POSTBOX: IBOX;
                  TeXFormat: boolean);

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  legend: string;
                  TexFormat: boolean);

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

  procedure PostFrame (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure PostVertAxis (var F: text;
                  XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

  procedure PostZeroPiAxis (var F: text;
                  var POSTBOX: IBOX);

  procedure PostFrameLite (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure PostPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostPlotMod (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PostConstant (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var constant: real);

  procedure PostSpikePlot (var F: text;
                  var lags: integer;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var spikeVec: vector);

  procedure ArrowDef (var F: text);

  procedure PoleCrossDef (var F: text);

  procedure ZeroCircDef (var F: text);

  procedure PostAargandHeader (var F: text;
                  TeXFormat: boolean);

  procedure PostAargand (var F: text;
                  duplex: boolean;
                  ARMASpec: ARMARecord);

  procedure PostAargandFooter (var F: text;
                  TeXFormat: boolean);

implementation

{________________________________________________}

  procedure PostHeader (var F: text;
                  var POSTBOX: IBOX;
                  TeXFormat: boolean);
    var
      BOXHGT, BOXWDT, INDENT: integer;
      LLx, LLy, URx, URy: integer;
      xOrigin, yOrigin: integer;

  begin
    xOrigin := 160;
    yOrigin := 560;

    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    if TeXFormat then
      Writeln(F, '\special{postscript')
    else
      begin
        LLx := xOrigin - 35;
        LLy := yOrigin - 25;
        URx := xOrigin + BOXWDT + 35;
        URy := yOrigin + BOXHGT + 10;
        Writeln(F, '%!PS-Adobe-2.0');
        Writeln(F, '%%Creator: SigmaPi');
        Writeln(F, '%%BoundingBox:', LLx : 3, '  ', LLy : 3, '  ', URx : 3, '  ', URy : 3);
      end;

    Writeln(F, '/showxhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '0  5 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '0 -20 rmoveto');
    Writeln(F, 'label dup stringwidth pop 2 div neg 0 rmoveto show} def');

    Writeln(F, '/showyhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y  moveto');
    Writeln(F, ' 5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '-15 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    Writeln(F, '/showqhatch');
    Writeln(F, '{/label exch def');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x  y moveto');
    Writeln(F, ' -5 0 rlineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y  moveto');
    Writeln(F, '30 -2 rmoveto');
    Writeln(F, 'label dup stringwidth pop neg 0 rmoveto show} def');

    if TeXFormat then
      begin {TeXFormat}
        INDENT := Round(((12 * 72) / 2.54 - BOXWDT) / 2);
        Writeln(F, INDENT : 2, '  ', -(BOXHGT + 15) : 2, '  translate');
      end {TeXFormat}
    else
      Writeln(F, xOrigin : 3, '  ', yOrigin : 3, ' translate ');

    Writeln(F, 'newpath');
    Writeln(F, '0.5 setlinewidth');

  end; {PostHeader}

{_________________________________________________}

  procedure PostFooter (var F: text;
                  var POSTBOX: IBOX;
                  legend: string;
                  TexFormat: boolean);

    var
      BOXHGT: integer;
      figNumberString: string;

  begin
{Writeln(F, 'stroke');}
    figNumberString := IntegerToString(figNumber);
    if TexFormat then
      begin
        Writeln(F, 'showpage}');
        BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
        Writeln(F, '\vbox to ', BOXHGT + 35 : 3, ' pt');
        Writeln(F, ' {\hrule width 12cm height 0cm depth 0cm \vfill}');
        Writeln(F, '\medskip');
        Writeln(F, '{\par\noindent{\bf Figure ', figNumber : 1, '.} ');
        Write(F, legend);
        Writeln(F, '\par}');
        Writeln(F, '\medskip');
      end
    else
      begin
        Writeln(F, 'showpage');
        Writeln(F, '%%Trailer');
        Writeln(F, '%%EOF');
        Writeln(F, '%% Figure ', fignumber : 1, '. ', legend);
      end;

    figNumber := figNumber + 1;

  end; {PostFooter}

{_________________________________________________}

  procedure PostMarks (var F: text;
                  XSTART, YSTART: real;
                  taly, firstMark, farBound, STEP: real;
                  Axis: string);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostMarks}

    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do

      begin {while toly}
        tolyString := TalyToString(toly);
        if Axis = 'xAxis' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showxhatch');
            HR := HR + STEP;
          end;

        if Axis = 'yAxis' then
          begin
            Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  (', tolyString, ') showyhatch');
            VT := VT + STEP;
          end;

        toly := toly + taly;
      end; {while TOLY}

  end; {PostMarks: Marks and Numbers}

{_________________________________________________}

  procedure PostFrame (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      sign: integer;
      XS, YS, XF, YF, BOXWDT, BOXHGT: real;
      FIRSTHATCH, XZERO, YZERO: real;
      HATCHMAX, HATCHMIN, VSTEP, HSTEP: real;
      RX, RY, rxPos, rxNeg, ryPos, ryNeg, taly: real;
      xMax, xMin, yMax, yMin, xRange: real;
      lowerBound, upperBound, boundRange, increment: real;
      nearBound, farBound, startMark: real;
      store: double;

  begin{PostFrame}
    xMax := realBOX.xMax;{the maximum value of x in the data vector}
    xMin := realBOX.xMin; {the minmum value  of x in the data vector}
    yMax := realBOX.yMax; {the maximum value of y in the Data vector}
    yMin := realBOX.yMin; {the minimum value of y in the Data vector}

    XS := POSTBOX.X1; {Left of the  Bounding Box}
    YS := POSTBOX.Y1; {Bottom of the Bounding Box}
    XF := POSTBOX.X2; {Right of the  Bounding Box}
    YF := POSTBOX.Y2; {Top of the Bounding Box}
    XZERO := XS; {Default for Graphics Origin}
    YZERO := YS; {Default for Graphics Origin}
    BOXWDT := XF - XS; {Width of Bounding Box}
    BOXHGT := YF - YS; { Height of Bounding Box}

    if POSTBOX.doMarks then
      begin
        Writeln(F, '/Symbol findfont');
        Writeln(F, '10 scalefont  setfont');
      end;
    if POSTBOX.doFrame then
      begin
        Writeln(F, XS : 5 : 2, '  ', YS : 5 : 2, '  moveto');
        Writeln(F, XS : 5 : 2, '  ', YF : 5 : 2, '  lineto');
        Writeln(F, XF : 5 : 2, '  ', YF : 5 : 2, '  lineto');
        Writeln(F, XF : 5 : 2, '  ', YS : 5 : 2, '  lineto');
        Writeln(F, 'closepath');
        Writeln(F, 'stroke');
        Writeln(F, 'newpath');
      end;

    if POSTBOX.yAxis then
      begin {Do the yAxis}

{...............................................................................................}

        if (yMin >= 0.0) and (yMax > 0.0) then
          begin {yMin = 0.0}
            if ((yMax - yMin) / yMax) < 0.3 then
              begin {small boundRange}
                lowerBound := yMin - 0.2 * (yMax - yMin);
                upperBound := yMax + 0.2 * (yMax - yMin);
                boundRange := upperBound - lowerBound;
                TallyPoints(boundRange, taly);
                lowerBound := Trunc(lowerBound / taly) * taly;
                boundRange := upperBound - lowerBound;
                YZERO := YS - lowerBound * (BOXHGT / boundRange);
              end {small boundRange}

            else{large boundRange}
              begin
                lowerBound := 0.0;
                upperBound := 1.2 * yMax;
                boundRange := upperBound - lowerBound;
                TallyPoints(boundRange, taly);
                YZERO := YS;
              end; {large boundRange}

            VSTEP := taly * (BOXHGT / boundRange);
            HSTEP := 0.0;
            if POSTBOX.doMarks then
              PostMarks(F, XS, YS, taly, lowerBound, upperBound, VSTEP, 'yAxis');

            HATCHMIN := YZERO;
            HATCHMAX := YS + Trunc(boundRange / taly) * VSTEP;
          end;{yMin >= 0.0 and yMax > 0.0}

{...............................................................................................}

        if (yMin < 0.0) and (yMax > 0.0) then
          begin {yMin < 0.0}
            increment := 0.1 * (yMax - yMin);
            if yMax > Abs(yMin) then
              begin
                ryPos := 1.2 * yMax;
                ryNeg := yMin - increment;
              end
            else
              begin
                ryNeg := 1.2 * yMin;
                ryPos := yMax + increment
              end;
            YZERO := YS - BOXHGT * ryNeg / (ryPos - ryNeg);

            if POSTBOX.doMarks and POSTBOX.xAxis then
              begin
                Writeln(F, XS : 5 : 2, '  ', YZERO : 5 : 2, '  moveto');
                Writeln(F, XF : 5 : 2, '  ', YZERO : 5 : 2, '  lineto')
              end;
            if ryPos > Abs(ryNeg) then
              begin
                TallyPoints(ryPos, taly);
                VSTEP := (YF - YZERO) * (taly / ryPos);
              end
            else {if ryPos < Abs(ryNeg)}
              begin
                RY := Abs(ryNeg);
                TallyPoints(RY, taly);
                VSTEP := (YZERO - YS) * (taly / RY);
              end;
            HSTEP := 0.0;
            if POSTBOX.doMarks then
              begin {doMarks}
                PostMarks(F, XS, YZERO, taly, 0.0, ryPos, VSTEP, 'yAxis');
                PostMarks(F, XS, YZERO, -taly, 0.0, ryNeg, -VSTEP, 'yAxis');
              end; {doMarks}
            HATCHMAX := YZERO + Trunc(ryPos / taly) * VSTEP;
            HATCHMIN := YZERO + Trunc(ryNeg / taly) * VSTEP;

            lowerBound := ryNeg;
            upperBound := ryPos;
          end; {yMin < 0.0 and yMax > 0.0}

{...............................................................................................}

        if (yMin < 0.0) and (yMax <= 0.0) then
          begin {yMin = 0.0}
            if ((yMin - yMax) / yMin) < 0.3 then
              begin {small boundRange}
                nearBound := yMax + 0.2 * (yMax - yMin);
                farBound := yMin - 0.2 * (yMax - yMin);
                boundRange := nearBound - farBound;
                TallyPoints(boundRange, taly);
                nearBound := Trunc(nearBound / taly) * taly;
                boundRange := nearBound - farBound;
                store := nearBound * (BOXHGT / boundRange);
                YZERO := YF + nearBound * (BOXHGT / boundRange);

              end {small boundRange}

            else{large boundRange}
              begin
                nearBound := 0.0;
                farBound := 1.2 * yMin;
                boundRange := nearBound - farBound;
                TallyPoints(boundRange, taly);
                YZERO := YF;
              end; {large boundRange}

            upperBound := nearBound;
            lowerBound := farBound;

            VSTEP := taly * (BOXHGT / boundRange);
            HSTEP := 0.0;
            PostMarks(F, XS, YF, -taly, nearBound, farBound, -VSTEP, 'yAxis');
            HATCHMIN := YZERO;
            HATCHMAX := YF - Trunc(boundRange / taly) * VSTEP;
          end;{yMin < 0.0 and yMax<= 0.0}

{...............................................................................................}

      end; {Do the yAxis}

    if POSTBOX.doMarks and (not POSTBOX.doFrame) then
      begin
        Writeln(F, XS : 5 : 2, '  ', HATCHMIN : 5 : 2, '  moveto');
        Writeln(F, XS : 5 : 2, '  ', HATCHMAX : 5 : 2, '  lineto');
      end;
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;

    if POSTBOX.xAxis then
      begin {Do the xAxis}

        if not POSTBOX.doFrame then
          begin
            Writeln(F, XS : 5 : 2, '  ', YS : 5 : 2, '  moveto');
            Writeln(F, XF : 5 : 2, '  ', YS : 5 : 2, '  lineto');
            Writeln(F, 'stroke');
            Writeln(F, 'newpath');
          end;

{...............................................................................................}

        if (xMin <= 0.0) and (xMax >= 0) then
          begin {xMin <= 0.0}
            xRange := xMax - xMin;
            rxPos := xMax;
            rxNeg := 0.0;
            if xMin < 0.0 then
              begin
                rxNeg := xMin;
                XZERO := XS - BOXWDT * rxNeg / (rxPos - rxNeg);
                Writeln(F, XZERO : 5 : 2, '  ', YS : 5 : 2, '  moveto');
                Writeln(F, XZERO : 5 : 2, '  ', YF : 5 : 2, '  lineto');
              end
            else
              XZERO := XS;

            if rxPos > Abs(rxNeg) then
              begin
                TallyPoints(rxPos, taly);
                HSTEP := (XF - XZERO) * (taly / rxPos);
              end
            else
              begin
                RX := Abs(rxNeg);
                TallyPoints(RX, taly);
                HSTEP := (XZERO - XS) * (taly / RX);
              end;
            VSTEP := 0;
            if xMax > 0.0 then
              PostMarks(F, XZERO, YS, taly, 0, rxPos, HSTEP, 'xAxis');
            if xMin < 0.0 then
              PostMarks(F, XZERO, YS, -taly, 0, rxNeg, -HSTEP, 'xAxis');
          end; {xMin <= 0.0 and xMax >= 0}

{...............................................................................................}

        if xMin * xMax > 0.0 then
          begin {xMin, xMax < 0 or xMin, xMax >0}
            sign := 1;
            if xMax < 0 then
              sign := -1;
            xRange := xMax - xMin;
            store := xMin * BOXWDT / xRange;
            XZERO := XS - xMin * BOXWDT / xRange;
            TallyPoints(xRange, taly);
            if xMin > 0 then
              begin
                farBound := xMax;
                nearBound := xMin;
              end;
            if xMax < 0 then
              begin
                farBound := xMin;
                nearBound := xMax;
              end;
            startMark := Trunc(nearBound / taly) * taly;
            if (Abs((startMark - nearBound) / nearBound) > 0.001) then
              startMark := startMark + sign * taly;
            HSTEP := (BOXWDT) * (taly / xRange);
            VSTEP := 0.0;
            if sign = 1 then
              begin
                FIRSTHATCH := XS + (startMark - xMin) * BOXWDT / xRange;
              end
            else
              begin
                FIRSTHATCH := XF - (xMax - startMark) * BOXWDT / xRange;
              end;
            PostMarks(F, FIRSTHATCH, YS, sign * taly, startMark, farBound, sign * HSTEP, 'xAxis');
          end;

{...............................................................................................}

        Writeln(F, 'stroke');
{Writeln(F, 'newpath');}

      end; {Do the xAxis}

    realBOX.xLB := xMin;
    realBOX.xRB := xMax;
    POSTBOX.X0 := Round(XZERO);
    POSTBOX.Y0 := Round(YZERO);

  end; {PostFrame: PostScript Framework}

{_________________________________________________}

  procedure PostVertAxis (var F: text;
                  XSTART, YSTART: integer;
                  taly, firstMark, farBound, VSTEP: real);

    var
      toly, HR, VT: real;
      tolyString: string;

  begin {PostVertAxis}

    toly := firstMark;
    HR := XSTART;
    VT := YSTART;

    while Abs(toly) <= Abs(farBound) do
      begin
        tolyString := TalyToString(toly);
        Writeln(F, HR : 3 : 2, '  ', VT : 3 : 2, '  (', tolyString, ') showyhatch');

        VT := VT + VSTEP;
        toly := toly + taly;
      end;

  end; {PostVertAxis}

{_________________________________________________}

  procedure PostZeroPiAxis (var F: text;
                  var POSTBOX: IBOX);

    var
      XR, YT, BOXWDT: real;

  begin{ZeroPiAxis}
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;

    XR := POSTBOX.X1;
    YT := POSTBOX.Y1; {type conversion}
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (0) showxhatch');

    XR := POSTBOX.X1 + BOXWDT / 4;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p/4) showxhatch');

    XR := POSTBOX.X1 + BOXWDT / 2;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p/2) showxhatch');

    XR := POSTBOX.X2 - BOXWDT / 4;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (3p/4) showxhatch');

    XR := POSTBOX.X2;
    Writeln(F, XR : 5 : 2, '  ', YT : 5 : 2, '  (p) showxhatch');
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

  end;{PostZeroPiAxis}

{_________________________________________________}

  procedure PostFrameLite (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX);

    var
      BOXHGT: integer;
      VSTEP, taly: real;
      yMax: real;
      lowerBound, upperBound: real;

  begin{PostFrameLite}
    yMax := realBOX.yMax;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    Writeln(F, '/Symbol findfont');
    Writeln(F, '10 scalefont  setfont');
    Writeln(F, POSTBOX.X1 : 2, '  ', POSTBOX.Y1 : 2, '  moveto');
    Writeln(F, POSTBOX.X1 : 2, '  ', POSTBOX.Y2 : 3, '  lineto');
    Writeln(F, POSTBOX.X2 : 3, '  ', POSTBOX.Y2 : 3, '  lineto');
    Writeln(F, POSTBOX.X2 : 2, '  ', POSTBOX.Y1 : 3, '  lineto');
    Writeln(F, 'closepath');
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    lowerBound := 0.0;
    upperBound := 1.2 * yMax;
    Tallypoints(upperBound, taly);

    VSTEP := taly * (BOXHGT / upperBound);

    PostVertAxis(F, POSTBOX.X0, POSTBOX.Y1, taly, lowerBound, upperBound, VSTEP);
    Writeln(F, 'stroke');
    Writeln(F, 'newpath');

    realBOX.yUB := upperBound;
    realBOX.yLB := lowerBound;
    realBOX.xLB := realBOX.xMin;
    realBOX.xRB := realBOX.xMax;

    POSTBOX.X0 := POSTBOX.X1;
    POSTBOX.Y0 := POSTBOX.Y1;

  end; {PostFrameLite}

{_________________________________________________}

  procedure PostPlot (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting an arbitrary number of data points over}
{the range defined by the width of the bounding box via the}
{index  i := 0 to Tcap - 1: Tcap := realBOX.NOPUNCT - 1}

    var
      i, Tcap: integer;
      XZERO, YZERO, BOXWDT, BOXHGT: real;
      HR, VT, xRange, yRange: real;

  begin
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    XZERO := POSTBOX.X0;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
    xRange := realBOX.NOPUNCT - 1; {real value}
    Tcap := realBOX.NOPUNCT - 1; {Integer Value}

    HR := BOXWDT;
    VT := YZERO + (dataVector[Tcap - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');

    for i := 0 to Tcap - 1 do
      begin {i}
        HR := XZERO + (i * BOXWDT / xRange);
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
        if (i mod 8 = 7) or (i = Tcap - 1) then
          Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2)
        else
          Write(F, HR : 3 : 2, ' ', VT : 3 : 2, ' ');
      end; {i}
    Writeln(F, Tcap : 3, ' {lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlot}

{_________________________________________________}

  procedure PostPlotMod (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var dataVector: longVector);

{This is for plotting a series of function evaluations that are equal}
{in number to the width to the bounding box measured in pixelpoints}
{via the index  i := 0 to BOXWDT - 1}

    var
      i, BOXWDT, BOXHGT, YZERO: integer;
      HR, VT, yRange: real;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    YZERO := POSTBOX.Y0;
    yRange := realBOX.yUB - realBOX.yLB;
 
    Writeln(F, 'newpath');
    Writeln(F, '/abscissa ', BOXWDT : 3, ' def');

    HR := BOXWDT;
    VT := YZERO + (dataVector[BOXWDT - 1] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');
{we are plotting in the reverse direction of x}

    for i := 0 to BOXWDT - 1 do
      begin {i}
        VT := YZERO + (dataVector[i] * BOXHGT / yRange);
{Here we  impose a limiting value on VT to keep the plot within the  bounding box}
        if VT > POSTBOX.Y2 then
          VT := POSTBOX.Y2;

        if (i mod 8 = 7) then
          Writeln(F, VT : 3 : 2) {this is to break the lines}
        else
          Write(F, VT : 3 : 2, ' ');
      end; {i}
    Write(F, BOXWDT : 3); {repeat these many times}
    Writeln(F, ' {/abscissa abscissa 1 sub def abscissa exch lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostPlotMod}

{________________________________________________}

  procedure PostConstant (var F: text;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var constant: real);

    var
      VT, BOXHGT, yRange: real;

  begin
    yRange := realBOX.yUB - realBOX.yLB;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    VT := POSTBOX.Y0 + constant * (BOXHGT / yRange);

    Writeln(F, 'newpath');
    Writeln(F, POSTBOX.X1 : 3, '  ', VT : 5 : 2, '  moveto');
    Writeln(F, POSTBOX.X2 : 3, '  ', VT : 5 : 2, '  lineto');
    Writeln(F, 'stroke');

  end; {PostConstant}

{_________________________________________________}

  procedure PostSpikePlot (var F: text;
                  var lags: integer;
                  var realBOX: rBOX;
                  var POSTBOX: IBOX;
                  var spikeVec: vector);

{This is for plotting a series of spikes}

    var
      i, BOXWDT, BOXHGT: integer;
      HR, VT, store, yRange, abscissa: real;

  begin
    yRange := realBOX.yUB - realBOX.yLB;
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;

    Writeln('newpath');
    Writeln(F, '2.0 setlinewidth');

    for i := 1 to lags do
      begin {i}
        abscissa := (i * BOXWDT) / (lags + 1);
        HR := POSTBOX.X0 + Round(abscissa);
        store := spikeVec[i] * (BOXHGT / yRange);
        VT := POSTBOX.Y0 + store;
        Writeln(F, HR : 5 : 2, '  ', POSTBOX.Y0 : 3, '  moveto');
        Writeln(F, HR : 5 : 2, '  ', VT : 5 : 2, '  lineto');
      end; {i}
    Writeln(F, 'stroke');

{Writeln('newpath');}
{Writeln(F, '0.5 setlinewidth');}

  end; {PostSpikePlot}

{________________________________________________}

  procedure ArrowDef (var F: text);

  begin
    Writeln(F, '/arrowdict 14 dict def');
    Writeln(F, 'arrowdict begin');
    Writeln(F, '/mtrx matrix def');
    Writeln(F, 'end');
    Writeln(F, '/arrow');
    Writeln(F, '{ arrowdict begin');
    Writeln(F, '/headlength exch def');
    Writeln(F, '/halfheadthickness exch 2 div def');
    Writeln(F, '/halfthickness exch 2 div def');
    Writeln(F, '/tipy exch def /tipx exch def');
    Writeln(F, '/taily exch def /tailx exch def');
    Writeln(F, '/dx tipx tailx sub def');
    Writeln(F, '/dy tipy taily sub def');
    Writeln(F, '/arrowlength dx dx mul dy dy mul add');
    Writeln(F, 'sqrt def');
    Writeln(F, '/angle dy dx atan def');
    Writeln(F, '/base arrowlength headlength sub def');
    Writeln(F, '/savematrix mtrx currentmatrix def');
    Writeln(F, 'tailx taily translate');
    Writeln(F, 'angle rotate');
    Writeln(F, '0 halfthickness neg moveto');
    Writeln(F, 'base halfthickness neg lineto');
    Writeln(F, 'base halfheadthickness neg lineto');
    Writeln(F, 'arrowlength 0 lineto');
    Writeln(F, 'base halfheadthickness lineto');
    Writeln(F, 'base halfthickness lineto');
    Writeln(F, '0 halfthickness lineto');
    Writeln(F, 'closepath');
    Writeln(F, 'savematrix setmatrix');
    Writeln(F, 'end');
    Writeln(F, '} def');
  end; {Arrow Def}

{_________________________________________________}

  procedure PoleCrossDef (var F: text);

  begin
    Writeln(F, '/cross{');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'x 4 sub y 4 add moveto');
    Writeln(F, '8 -8  rlineto');
    Writeln(F, 'x 4 sub y 4 sub moveto');
    Writeln(F, '8  8  rlineto');
    Writeln(F, '} def');

    Writeln(F, '/polecross');
    Writeln(F, '{ /modulus exch def');
    Writeln(F, '/argument exch def');
    Writeln(F, 'gsave');
    Writeln(F, 'newpath');
    Writeln(F, '0 0 moveto');
    Writeln(F, 'argument rotate');
    Writeln(F, 'modulus 0 rlineto');
    Writeln(F, 'currentpoint cross');
    Writeln(F, 'stroke');
    Writeln(F, 'grestore');
    Writeln(F, '}def');
  end; {PoleCross Def}

{_________________________________________________}

  procedure ZeroCircDef (var F: text);

  begin
    Writeln(F, '/circ{');
    Writeln(F, '/y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, '1 setgray');
    Writeln(F, 'x   y  2.5  0 360 arc ');
    Writeln(F, 'fill');
    Writeln(F, '0 setgray');
    Writeln(F, 'x   y   2.5  0 360 arc ');
    Writeln(F, '} def');

    Writeln(F, '/zerocirc');
    Writeln(F, '{ /y exch def');
    Writeln(F, '/x exch def');
    Writeln(F, 'newpath');
    Writeln(F, '0  0 moveto');
    Writeln(F, 'x  y  lineto');
    Writeln(F, 'stroke');
    Writeln(F, 'x  y circ ');
    Writeln(F, 'stroke');
    Writeln(F, '}def');
  end; {ZeroCirc Def}

{_________________________________________________}

  procedure PostAargandHeader (var F: text;
                  TeXFormat: boolean);

  begin
    if TeXFormat then
      Writeln(F, '\special{postscript')
    else
      begin {EPS format}
        Writeln(F, '%!PS-Adobe-2.0');
        Writeln(F, '%%Creator: SigmaPi');
        Writeln(F, '%%BoundingBox:95  567  415  740');
{is this appropriate both for singke and duplex diagrams?}
      end; {EPS format}

    ArrowDef(F);
    PoleCrossDef(F);
    ZeroCircDef(F);

  end;{PostAargandHeader}

{_________________________________________________}

  procedure PostAargand (var F: text;
                  duplex: boolean;
                  ARMASpec: ARMARecord);

{This procedure requires that the poles and zersos should be provided in Cartesian form}

    var
      p, q, i: integer;
      argument: integer;
      xRayTip, yRayTip, modulus, radToDeg: real;
      poles, zeros: complexVector;

  begin {PostAargand}
    p := ARMASpec.p;
    q := ARMASpec.q;

    poles := ARMASpec.poles;
    zeros := ARMASpec.zeros;

    radToDeg := 45 / Arctan(1.0);

    Writeln(F, '0.5 setlinewidth');
    Writeln(F, 'newpath');

 {Scribe the unit circle and the axis arrows}
    Writeln(F, '0 0 1 units 0 360 arc');
    Writeln(F, 'stroke');

 {Scribe the legend -i}
    Writeln(F, '15 neg 1 neg units 12 sub moveto');
    Writeln(F, '/Symbol findfont 10 scalefont setfont');
    Writeln(F, '(-) show 1 0 rmoveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(i) show');

 {Scribe the legend i}
    Writeln(F, '8 neg 1 units 5 add moveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(i) show');

  {Scribe the legend -1}
    Writeln(F, '1 neg units 15 sub 12 neg  moveto');
    Writeln(F, '/Symbol findfont 10 scalefont setfont');
    Writeln(F, '(-1) show');

 {Scribe the legend 1}
    Writeln(F, '1  units 4 add 12 neg  moveto');
    Writeln(F, '/Symbol findfont 10 scalefont setfont');
    Writeln(F, '(1) show');

    Writeln(F, 'newpath');
    Writeln(F, '/tailthick 0.5  def');
    Writeln(F, '/headthick 4  def');
    Writeln(F, '/headlength  6 def');

    Writeln(F, '/tailx 1.5 neg units  def');
    Writeln(F, '/taily  0 units  def');
    Writeln(F, '/tipx  1.5 units  def');
    Writeln(F, '/tipy  0 units  def');
    Writeln(F, 'tailx taily tipx tipy tailthick headthick headlength arrow');
    Writeln(F, 'fill');

 {Scribe the legend Re}
    Writeln(F, 'tipx 6 add tipy 3 sub moveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(Re) show');

    Writeln(F, 'newpath');
    Writeln(F, '/tailx  0  units  def');
    Writeln(F, '/taily  1.5 neg units  6 add  def');
    Writeln(F, '/tipx  0  units  def');
    Writeln(F, '/tipy  1.5 units def');
    Writeln(F, 'tailx taily tipx tipy tailthick headthick headlength arrow');
    Writeln(F, 'fill');

 {Scribe the legend Im}
    Writeln(F, 'tipx 9 add tipy 4 sub moveto');
    Writeln(F, '/Times-Italic findfont 10 scalefont setfont');
    Writeln(F, '(Im) show');

 {------------------------------------}

 {Construct the rays}
    for i := 1 to p do
      begin
        modulus := TheModulus(poles[i]);
        argument := Round(TheArgument(poles[i]) * radToDeg);
        Writeln(F, 'newpath');
        Writeln(F, '0  0 moveto');
        Writeln(F, argument : 3, '  ', modulus : 2 : 2, ' units  polecross');
        Writeln(F, 'stroke');
      end;

    for i := 1 to q do
      begin
        xRayTip := zeros[i].re;
        yRayTip := zeros[i].im;
        Writeln(F, 'newpath');
        Writeln(F, '0  0 moveto');
        Writeln(F, xRayTip : 2 : 2, ' units ', yRayTip : 2 : 2, ' units  zerocirc');
        Writeln(F, 'stroke');
      end;

  end; {PostAargand }

{_________________________________________________}

  procedure PostAargandFooter (var F: text;
                  TeXFormat: boolean);

  begin
 {Writeln(F, 'stroke');}
    if TeXFormat then
      begin
        Writeln(F, 'showpage}');
        Writeln(F, '\vbox to  180 pt');
        Writeln(F, '{\hrule width 342pt height 0pt depth 0pt \vfill}');
      end
    else
      begin
        Writeln(F, 'showpage');
        Writeln(F, '%%Trailer');
        Writeln(F, '%%EOF');
      end;

  end;{PostAargandFooter}

{_________________________________________________}

end. {POST: Unit 6 of OVERSAMPLE.PAS}

