unit SPECTRAL; {Unit 7 of OVERSAMPLE.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, Graph, Dos, Crt;


  procedure BighamFFT (var xReal, xImag: longVector;
                  N, NU: integer);

  procedure SlowCosine (var gamma, spectrum: longVector;
                  var Tcap: integer);

  procedure SlowInverseCosine (var omegaFactor: real;
                  var gamma, spectrum: longVector;
                  var Ncap, lags: integer);

  procedure OverSampledCovars (var gamma: vector;
                  var spectrum: longVector;
                  var Ncap, lags, sampleRate: integer);

  procedure Laurent (var mu, gamma: vector;
                  q: integer);

  function Spectrum (var ARMASpec: ARMARecord;
                  var scale: real;
                  var alphaLaurent, muLaurent: vector;
                  omega: real): real;

  procedure MakeSpectrumVec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure MakeScreenBandSpec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

  procedure MakePostBandSpec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var specVec: longVector;
                  var bandSpecBOX: rBOX);

  procedure ScreenBandPlot (var ARMASpec: ARMARecord;
                  var realBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var dataVector: longVector);

  procedure LimitLine (var SCREENBOX: IBOX;
                  var ARMASpec: ARMARecord);

  procedure PostBandPlot (var F: text;
                  var ARMASpec: ARMARecord;
                  var bandSpecBOX: rBOX;
                  var POSTBOX: IBOX;
                  var specVec: longVector);

  procedure PostLimitLine (var F: text;
                  var POSTBOX: IBOX;
                  var ARMASpec: ARMARecord);

  procedure PlotLittoralSpectrum (var ARMASpec: ARMARecord;
                  var spectrumBOX: rBOX;
                  var SCREENBOX: IBOX);

  procedure PostLittoralSpectrum (var F: text;
                  var ARMASpec: ARMARecord;
                  var spectrumBOX: rBOX;
                  var POSTBOX: IBOX);

  procedure MakeNoisySpecVec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var noisySpecVec: longVector;
                  var spectrumBOX: rBOX);

implementation

{_________________________________________________}

  function IBITR (J, NU: integer): integer;
    var
      I, J1, J2, K: integer;

  begin {IBITR}
    J1 := J;
    K := 0;
    for I := 1 to NU do
      begin
        J2 := J1 div 2;
        K := K * 2 + (J1 - 2 * J2);
        J1 := J2
      end;
    IBITR := K
  end; {IBITR}

{________________________________________________}

  procedure BighamFFT (var xReal, xImag: longVector;
                  N, NU: integer);
    label
      999;

    var
      N2, NU1, I, L, K, M: integer;
      TREAL, TIMAG, P, arg, cosine, sine: real;

  begin
    N2 := N div 2;
    NU1 := NU - 1;
    k := 0;
    for L := 1 to NU do
      begin
999:
        for I := 1 to N2 do
          begin
            M := k div Round(Exp(NU1 * LN(2)));
            P := IBITR(M, NU);
            arg := 6.283185 * P / N;
            cosine := Cos(arg);
            sine := Sin(arg);
            TREAL := xReal[K + N2] * cosine + xImag[K + N2] * sine;
            TIMAG := xImag[k + N2] * cosine - xReal[k + N2] * sine;
            xReal[k + N2] := xReal[k] - TREAL;
            xImag[K + N2] := xImag[k] - TIMAG;
            xReal[k] := xReal[k] + TREAL;
            xImag[K] := xImag[k] + TIMAG;
            k := k + 1
          end;
        k := k + N2;
        if k < N then
          goto 999;
        K := 0;
        NU1 := NU1 - 1;
        N2 := N2 div 2
      end;

    for K := 0 to N - 1 do
      begin
        I := IBITR(K, NU);
        if I > K then
          begin
            TREAL := xReal[K];
            TIMAG := xImag[K];
            xReal[K] := xReal[I];
            xImag[K] := xImag[I];
            xReal[I] := TREAL;
            xImag[I] := TIMAG;
          end;
      end;
  end; {BinghamFFT}

{_________________________________________________}

  procedure SlowCosine (var gamma, spectrum: longVector;
                  var Tcap: integer);
    var
      t, j, NCap: integer;
      twoPi, piOverN, omegaj, sign: real;

  begin
    Ncap := Tcap div 2;
    twoPi := 8.0 * Arctan(1.0);
    piOverN := twoPi / Tcap;
    sign := 1.0;
    for j := 0 to Ncap do
      begin {j}
        omegaj := piOverN * j;
        spectrum[j] := gamma[0];
        for t := 1 to Ncap - 1 do
          spectrum[j] := spectrum[j] + 2 * gamma[t] * cos(omegaj * t);
        spectrum[j] := spectrum[j] + sign * gamma[Ncap];
        sign := -sign;
      end{j}

  end; {SlowCosine}
{_________________________________________________}

  procedure SlowInverseCosine (var omegaFactor: real;
                  var gamma, spectrum: longVector;
                  var Ncap, lags: integer);
    var
      t, j: integer;
      pi, piOverN, increment, omegat, sign: real;

  begin
    pi := 4.0 * Arctan(1.0);
    piOverN := pi / Ncap;
{increment := (piOverN * (lags + 1)) / Ncap;}
    increment := piOverN * omegaFactor;

{In the ordinary cosine FT, we should have omegaFactor =1}

    gamma[0] := spectrum[0];
    for j := 1 to Ncap - 1 do {Treating gamma0 as a special case}
      gamma[0] := gamma[0] + 2 * spectrum[j];
    gamma[0] := gamma[0] + spectrum[Ncap];

    sign := -1.0;
    for t := 1 to Ncap do {Ncap is the pixel width ofthe graphic}
      begin {t}
        omegat := increment * t;
        gamma[t] := spectrum[0];
        for j := 1 to Ncap - 1 do
          gamma[t] := gamma[t] + 2 * spectrum[j] * cos(omegat * j);
        gamma[t] := gamma[t] + sign * spectrum[Ncap];
        sign := -sign;
        gamma[t] := gamma[t] / gamma[0]
      end;{t}
    gamma[0] := 1.0;

  end; {SlowInverseCosine}

{_________________________________________________}

  procedure OverSampledCovars (var gamma: vector;
                  var spectrum: longVector;
                  var Ncap, lags, sampleRate: integer);
    var
      pi, increment, omegak, sign: real;
      j, k: integer;

  begin

    pi := 4.0 * Arctan(1.0);
    increment := pi / (Ncap * sampleRate);

    gamma[0] := spectrum[0];
    for j := 1 to Ncap - 1 do {Treating gamma0 as a special case}
      gamma[0] := gamma[0] + 2 * spectrum[j];
    gamma[0] := gamma[0] + spectrum[Ncap];

    sign := -1.0;
    for k := 1 to lags do {lags is the number ordinates}
      begin {k}
        omegak := increment * k;
        gamma[k] := spectrum[0];
        for j := 1 to Ncap - 1 do
          gamma[k] := gamma[k] + 2 * spectrum[j] * cos(omegak * j);
        gamma[k] := gamma[k] + sign * spectrum[Ncap];
        sign := -sign;
        gamma[k] := gamma[k] / gamma[0];
      end;{k}
    gamma[0] := 1.0;

  end; {OverSampledCovars}

{_________________________________________________}

  procedure Laurent (var mu, gamma: vector;
                  q: integer);

    var
      i, j: integer;

  begin
    for i := 0 to q do
      begin {i}
        gamma[i] := 0.0;
        for j := 0 to q - i do
          gamma[i] := gamma[i] + mu[j] * mu[j + i];
      end; {i}
  end; {Laurent}

{________________________________________________}

  function Spectrum (var ARMASpec: ARMARecord;
                  var scale: real;
                  var alphaLaurent, muLaurent: vector;
                  omega: real): real;

    var
      j: integer;
      numerator, denominator: extended;
      twoPi, flatSpectrum, specOrdinate: real;

  begin
    twoPi := 8.0 * Arctan(1.0);

    with ARMASpec do
      begin
        numerator := muLaurent[0];
        for j := 1 to q do
          numerator := numerator + 2 * muLaurent[j] * Cos(omega * j);
        denominator := alphaLaurent[0];
        for j := 1 to p do
          denominator := denominator + 2 * alphaLaurent[j] * Cos(omega * j);
        specOrdinate := (numerator / denominator) * scale; {scale := sigmaSqr/twoPi}

        if contaminated then
          begin
            flatSpectrum := noiseVar / twoPi;
            Spectrum := specOrdinate + flatSpectrum;
          end
        else
          Spectrum := specOrdinate;
      end; {with ARMASpec}

  end; {Spectrum}

{_________________________________________________}

  procedure MakeSpectrumVec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

{This is the data-generating spectrum supported on the interval [0, pi]}
{The routine serves to generate a vector of points that is used both in }
{the screen rendition and in the PostScript rendition}

    var
      j, BOXWDT: integer;
      pi, twoPi, increment, omega, scale: real;
      alphaLaurent, muLaurent: vector;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

    pi := 4.0 * Arctan(1.0);
    twoPi := 2.0 * pi;
    scale := ARMASpec.varEpsilon / twoPi;

    Laurent(ARMASpec.mu, muLaurent, ARMASpec.q);
    Laurent(ARMASpec.alpha, alphaLaurent, ARMASpec.p);

    spectrumBOX.xMax := pi; {the maximum value of x in the data vector}
    spectrumBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    spectrumBOX.xLB := spectrumBOX.xMin;
    spectrumBOX.xRB := spectrumBOX.xMax;
    spectrumBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    spectrumBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    spectrumBOX.NOPUNCT := BOXWDT;

    omega := 0.0;
    increment := pi / BOXWDT;

    for j := 0 to BOXWDT do
      begin {j}
        spectrumVec[j] := Spectrum(ARMASpec, scale, alphaLaurent, muLaurent, omega);
        if spectrumVec[j] > spectrumBOX.yMax then
          spectrumBOX.yMax := spectrumVec[j];
        omega := omega + increment;
      end; {j}

    ARMASpec.spectrumFound := true;

  end;{MakeSpectrumVec}

{________________________________________________}

  procedure MakeScreenBandSpec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector;
                  var spectrumBOX: rBOX);

    var
      j, BOXWDT, SPECWDT: integer;
      pi, twoPi, increment, omega, scale: real;
      alphaLaurent, muLaurent: vector;

  begin

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    SPECWDT := Round((BOXWDT * ARMASpec.omega2Deg) / 180);

    pi := 4.0 * Arctan(1.0);
    twoPi := 2.0 * pi;

    scale := ARMASpec.varEpsilon / twoPi;
    scale := (scale * 180) / ARMASpec.omega2Deg;

    Laurent(ARMASpec.mu, muLaurent, ARMASpec.q);
    Laurent(ARMASpec.alpha, alphaLaurent, ARMASpec.p);

    spectrumBOX.xMax := pi; {the maximum value of x in the data vector}
    spectrumBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    spectrumBOX.xLB := spectrumBOX.xMin;
    spectrumBOX.xRB := spectrumBOX.xMax;
    spectrumBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    spectrumBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    spectrumBOX.NOPUNCT := BOXWDT;

    omega := 0.0;
    increment := pi / SPECWDT;

    for j := 0 to SPECWDT do
      begin {j}
        spectrumVec[j] := Spectrum(ARMASpec, scale, alphaLaurent, muLaurent, omega);
        if spectrumVec[j] > spectrumBOX.yMax then
          spectrumBOX.yMax := spectrumVec[j];
        omega := omega + increment;
      end; {j}

  end; {MakeScreenBandSpec}

{________________________________________________}

  procedure MakePostBandSpec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var specVec: longVector;
                  var bandSpecBOX: rBOX);

{This procedure generates a set of points on a finer grid than does the}
{corresponding screen graphics procedure MakeScreenBandSpec,}

    var
      j, BOXWDT, SPECWDT, resolution: integer;
      pi, twoPi, increment, omega, scale, bandFraction: real;
      alphaLaurent, muLaurent: vector;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    SPECWDT := Round((BOXWDT * ARMASpec.omega2Deg) / 180);
    bandFraction := ARMASpec.omega2Deg / 180;

    resolution := 1;
    if bandFraction < 0.5 then
      resolution := 2;
    if bandFraction < 0.25 then
      resolution := 4;

    pi := 4.0 * Arctan(1.0);
    twoPi := 2.0 * pi;

    scale := ARMASpec.varepsilon / twoPi;
    scale := scale * 180 / ARMASpec.omega2Deg;

    Laurent(ARMASpec.mu, muLaurent, ARMASpec.q);
    Laurent(ARMASpec.alpha, alphaLaurent, ARMASpec.p);

    bandSpecBOX.xMax := pi; {the maximum value of x in the data vector}
    bandSpecBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    bandSpecBOX.xLB := spectrumBOX.xMin;
    bandSpecBOX.xRB := spectrumBOX.xMax;
    bandSpecBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    bandSpecBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    bandSpecBOX.NOPUNCT := BOXWDT;

    omega := 0.0;
    increment := pi / (SPECWDT * resolution); {Diminish this to give more resolution}

    for j := 0 to SPECWDT * resolution do
      begin {j}
        specVec[j] := Spectrum(ARMASpec, scale, alphaLaurent, muLaurent, omega);
        if specVec[j] > bandSpecBOX.yMax then
          bandSpecBOX.yMax := specVec[j];
        omega := omega + increment;
      end; {j}

    bandSpecBOX.NOPUNCT := SPECWDT * resolution; {This will get used}

  end; {MakePostBandSpec}

{_________________________________________________}

  procedure ScreenBandPlot (var ARMASpec: ARMARecord;
                  var realBOX: rBOX; {spectrumBOX}
                  var SCREENBOX: IBOX;
                  var dataVector: longVector); {limSpecVec}

{This is for plotting a number SPEWDT of data points over}
{the range defined by the  width of a band-limited spectrum}

    var
      i, HR, VT, BOXHGT, BOXWDT, SPECWDT: integer;
      yRange: real;
      store: double;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    SPECWDT := Round((BOXWDT * ARMASpec.omega2Deg) / 180);

    yRange := realBOX.yUB - realBOX.yLB;
 
    for i := 0 to SPECWDT do
      begin {i}
        HR := SCREENBOX.X1 + i;
        store := (dataVector[i] - realBOX.yLB) / yRange;
        VT := Round(SCREENBOX.Y2 - store * BOXHGT);
        if VT < SCREENBOX.Y1 then
          VT := SCREENBOX.Y1;
        if i = 0 then
          MoveTo(HR, VT)
        else
          LineTo(HR, VT);
      end; {i}

  end; {ScreenBandPlot}

{________________________________________________}

  procedure LimitLine (var SCREENBOX: IBOX;
                  var ARMASpec: ARMARecord);

    var
      BOXWDT, SPECWDT: integer;

  begin
    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    SPECWDT := Round((BOXWDT * ARMASpec.omega2Deg) / 180);
    SPECWDT := SPECWDT + SCREENBOX.X1;
    MoveTo(SPECWDT, SCREENBOX.Y1);
    LineTo(SPECWDT, SCREENBOX.Y2);
  end; {LimitLine}

{________________________________________________}

  procedure PostBandPlot (var F: text;
                  var ARMASpec: ARMARecord;
                  var bandSpecBOX: rBOX;
                  var POSTBOX: IBOX;
                  var specVec: longVector);
    var
      j, BOXWDT, SPECWDT, BOXHGT, YZERO: integer;
      HR, VT, yRange, bandFraction, decrement: real;

  begin

    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    YZERO := POSTBOX.Y0;
    SPECWDT := Round((BOXWDT * ARMASpec.omega2Deg) / 180);
{this is the width in points of the spectral support}

    yRange := bandSpecBOX.yUB - bandSpecBOX.yLB; {where do these come from?}

    bandFraction := ARMASpec.omega2Deg / 180;
    if bandFraction < 0.5 then
      decrement := 0.5;
    if bandFraction < 0.25 then
      decrement := 0.25;

    Writeln(F, 'newpath');
    Writeln(F, '/abscissa ', SPECWDT : 3, ' def');

    HR := SPECWDT;
    VT := (specVec[bandSpecBOX.NOPUNCT] * BOXHGT / yRange);
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');

    for j := 0 to bandSpecBOX.NOPUNCT - 1 do
      begin {j}
        VT := YZERO + (specVec[j] * BOXHGT / yRange);
        if (j mod 8 = 7) or (j = bandSpecBOX.NOPUNCT - 1) then
          Writeln(F, VT : 3 : 2)
        else
          Write(F, VT : 3 : 2, ' ');
      end; {j}

    Write(F, bandSpecBOX.NOPUNCT : 3);
{This should correspond to the number of points at which}
{the band-limited spectrum has been evaluated}

    Writeln(F, ' {/abscissa abscissa ', decrement : 1 : 2, ' sub def abscissa exch lineto} repeat');
    Writeln(F, 'stroke');

  end; {PostBandPlot}

 {________________________________________________}

  procedure PostLimitLine (var F: text;
                  var POSTBOX: IBOX;
                  var ARMASpec: ARMARecord);

    var
      BOXWDT: integer;
      SPECWDT, HR, VT: real;

  begin
    BOXWDT := POSTBOX.X2 - POSTBOX.X1;
    SPECWDT := BOXWDT * ARMASpec.omega2Deg / 180.0;

    Writeln(F, '0.5 setlinewidth');
    Writeln(F, 'newpath');
    HR := SPECWDT;
    VT := POSTBOX.Y1;
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' moveto');
    VT := POSTBOX.Y2;
    Writeln(F, HR : 3 : 2, ' ', VT : 3 : 2, ' lineto');
    Writeln(F, 'stroke');

  end; {PostLimitLine}

 {________________________________________________}

  procedure PlotLittoralSpectrum (var ARMASpec: ARMARecord;
                  var spectrumBOX: rBOX;
                  var SCREENBOX: IBOX);
    var
      HR1, HR2, VT, BOXHGT: integer;
      twoPi, yRange, vert: real;

  begin
    twoPi := 8.0 * ArcTan(1.0);
    HR1 := SCREENBOX.X1;
    HR2 := SCREENBOX.X2;
    BOXHGT := SCREENBOX.Y2 - SCREENBOX.Y1;
    yRange := spectrumBOX.yUB - spectrumBOX.yLB;
    vert := (BOXHGT * ARMASpec.noiseVar) / twoPi;
    VT := SCREENBOX.Y2 - Round(vert / yRange);
    {SetDash;}
    Moveto(HR1, VT);
    LineTo(HR2, VT);
    {PenNormal;}

  end; {PlotLittoralSpectrum}

{________________________________________________}

  procedure PostLittoralSpectrum (var F: text;
                  var ARMASpec: ARMARecord;
                  var spectrumBOX: rBOX;
                  var POSTBOX: IBOX);
    var
      BOXHGT: integer;
      HR1, HR2, VT, twoPi, yRange, vert: real;

  begin
    twoPi := 8.0 * ArcTan(1.0);
    HR1 := POSTBOX.X1;
    HR2 := POSTBOX.X2;
    BOXHGT := POSTBOX.Y2 - POSTBOX.Y1;
    yRange := spectrumBOX.yUB - spectrumBOX.yLB;
    vert := (BOXHGT * ARMASpec.noiseVar) / twoPi;
    VT := POSTBOX.Y0 + (vert / yRange);

    Writeln(F, 'newpath');
    Writeln(F, ' [ 4 2 ] 0 setdash ');
    Writeln(F, HR1 : 3 : 2, ' ', VT : 3 : 2, ' moveto');
    Writeln(F, HR2 : 3 : 2, ' ', VT : 3 : 2, ' lineto');
    Writeln(F, 'stroke');
    Writeln(F, ' [] 0 setdash ');

  end; {PostLittoralSpectrum}

{________________________________________________}

  procedure MakeNoisySpecVec (var ARMASpec: ARMARecord;
                  var SCREENBOX: IBOX;
                  var noisySpecVec: longVector;
                  var spectrumBOX: rBOX);

    var
      j, BOXWDT, SHORTBOX: integer;
      pi, twoPi, increment, omega, scale, flatSpectrum, realBOX: real;
      alphaLaurent, muLaurent: vector;

  begin

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;
    realBOX := BOXWDT / ARMASpec.sampleRate;
    SHORTBOX := Round(realBOX);

    pi := 4.0 * Arctan(1.0);

    twoPi := 2.0 * pi;
    scale := ARMASpec.varEpsilon / twoPi;

    Laurent(ARMASpec.mu, muLaurent, ARMASpec.q);
    Laurent(ARMASpec.alpha, alphaLaurent, ARMASpec.p);

    spectrumBOX.xMax := pi; {the maximum value of x in the data vector}
    spectrumBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    spectrumBOX.xLB := spectrumBOX.xMin;
    spectrumBOX.xRB := spectrumBOX.xMax;
    spectrumBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    spectrumBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    spectrumBOX.NOPUNCT := BOXWDT;

    omega := 0.0;
    increment := pi / SHORTBOX;

    for j := 0 to SHORTBOX do
      begin {j}
        noisySpecVec[j] := Spectrum(ARMASpec, scale, alphaLaurent, muLaurent, omega);
        if noisySpecVec[j] > spectrumBOX.yMax then
          spectrumBOX.yMax := noisySpecVec[j];
        omega := omega + increment;
      end; {j}

    flatSpectrum := ARMASpec.noiseVar / twoPi;
    for j := SHORTBOX + 1 to BOXWDT do
      noisySpecVec[j] := flatSpectrum;

    ARMASpec.spectrumFound := true;

  end; {MakeNoisySpecVec }

{_________________________________________________}

end. {SPECTRAL: Unit 7 of OVERSAMPLE.PAS}

